@@ -38,7 +38,9 @@ if ($module_name ne 'htaccess') {
 		# Need to build list of supported modules
 		local ($ver, $mods, $fullver) = &httpd_info($httpd);
 		if ($ver) {
-			local @mods = map { "$_/$ver" } &configurable_modules();
+			my @allmods = &available_modules();
+			local @mods = map { "$_/$ver" }
+				          &configurable_modules(\@allmods);
 			foreach my $m (@mods) {
 				if ($m =~ /(\S+)\/(\S+)/) {
 					$httpd_modules{$1} = $2;
@@ -47,10 +49,14 @@ if ($module_name ne 'htaccess') {
 			# Call again now that known modules have been set, as
 			# sometimes there are dependencies due to LoadModule
 			# statements in an IfModule block
-			@mods = map { "$_/$ver" } &configurable_modules();
+			undef(@get_config_cache);
+			@allmods = &available_modules();
+			@mods = map { "$_/$ver" }
+				    &configurable_modules(\@allmods);
 			local %site = ( 'size' => $st[7],
 					'path' => $httpd,
 					'modules' => join(' ', @mods),
+					'allmodules' => join(' ', @allmods),
 					'version' => $ver,
 					'fullversion' => $fullver,
 					'webmin' => &get_webmin_version() );
@@ -72,6 +78,9 @@ if (&read_file($site_file, \%site)) {
 			$httpd_modules{$1} = $2;
 			}
 		}
+	foreach $m (split(/\s+/, $site{'allmodules'})) {
+		$all_httpd_modules{$m} = $site{'version'};
+		}
 	foreach $m (keys %httpd_modules) {
 		if (!-r "$module_root_directory/$m.pl") {
 			delete($httpd_modules{$m});
@@ -540,7 +549,7 @@ local($i, @old, $lref, $change, $len, $v);
 @old = &find_directive_struct($_[0], $_[2]);
 local @files;
 for($i=0; $i<@old || $i<@{$_[1]}; $i++) {
-	$v = ${$_[1]}[$i];
+	$v = $i<@{$_[1]} ? $_[1]->[$i] : undef;
 	if ($i >= @old) {
 		# a new directive is being added. If other directives of this
 		# type exist, add it after them. Otherwise, put it at the end of
@@ -571,7 +580,8 @@ for($i=0; $i<@old || $i<@{$_[1]}; $i++) {
 			# in this section
 			local($f, %v, $j);
 			$f = $_[2]->[0]->{'file'};
-			for($j=0; $_[2]->[$j]->{'file'} eq $f; $j++) { }
+			for($j=0; $j < @{$_[2]} &&
+				  $_[2]->[$j]->{'file'} eq $f; $j++) { }
 			$lref = &read_file_lines($f);
 			if ($_[2] eq $_[3]) {
 				# Top-level, so add to the end of the file
@@ -645,16 +655,19 @@ if ($newdir) {
 	if ($isrc) {
 		&recursive_set_indent($newdir, $isrc->{'indent'});
 		}
+	$newdir{'words'} = &wsplit($newdir{'value'});
 	@newlines = &directive_lines($newdir);
 	}
 if ($olddir && $newdir) {
 	# Update in place
+	$newdir->{'words'} = &wsplit($newdir->{'value'});
 	if ($first) {
 		# Just changing first and last line, like virtualhost IP
 		$lref->[$olddir->{'line'}] = $newlines[0];
 		$lref->[$olddir->{'eline'}] = $newlines[$#newlines];
 		$olddir->{'name'} = $newdir->{'name'};
 		$olddir->{'value'} = $newdir->{'value'};
+		$olddir->{'words'} = $newdir->{'words'};
 		}
 	else {
 		# Re-writing whole block
@@ -698,6 +711,7 @@ elsif (!$olddir && $newdir) {
 		$addpos--;
 		$addline = $pconf->[$addpos]->{'eline'}+1;
 		}
+	$newdir->{'words'} = &wsplit($newdir->{'value'});
 	$newdir->{'file'} = $file;
 	$newdir->{'line'} = $addline;
 	$newdir->{'eline'} = $addline + scalar(@newlines) - 1;
@@ -739,13 +753,11 @@ foreach my $dir (@$dirs) {
 	if ($dir->{'type'}) {
 		# Do sub-members too
 		&recursive_set_lines_files($dir->{'members'}, $line+1, $file);
-		$line += scalar(@{$dir->{'members'}})+1;
-		$dir->{'eline'} = $line;
+		$line += scalar(grep { $_->{'name'} ne 'dummy' }
+				     @{$dir->{'members'}})+1;
 		}
-	else {
-		$dir->{'eline'} = $line;
-		}
-	$line++;
+	$dir->{'eline'} = $line;
+	$line++ if ($dir->{'name'} ne 'dummy');
 	}
 return $line;
 }
@@ -769,17 +781,17 @@ unlink($file);
 # beyond the given line.
 sub renumber
 {
-local($d);
-if (!$_[3]) { return; }
-foreach $d (@{$_[0]}) {
-	if ($d->{'file'} eq $_[2] && $d->{'line'} >= $_[1]) {
-		$d->{'line'} += $_[3];
+my ($conf, $line, $file, $offset) = @_;
+return if (!$offset);
+foreach my $d (@$conf) {
+	if ($d->{'file'} eq $file && $d->{'line'} >= $_[1]) {
+		$d->{'line'} += $offset;
 		}
-	if ($d->{'file'} eq $_[2] && $d->{'eline'} >= $_[1]) {
-		$d->{'eline'} += $_[3];
+	if ($d->{'file'} eq $file && $d->{'eline'} >= $_[1]) {
+		$d->{'eline'} += $offset;
 		}
 	if ($d->{'type'}) {
-		&renumber($d->{'members'}, $_[1], $_[2], $_[3]);
+		&renumber($d->{'members'}, $line, $file, $offset);
 		}
 	}
 }
@@ -817,21 +829,21 @@ return $_[0] ? $_[0] : $_[1];
 }
 
 # make_directives(ref, version, module)
+# Return directives suitable for this system and version
 sub make_directives
 {
-local(@rv, $aref);
-$aref = $_[0];
-local $ver = $_[1];
+my ($aref, $ver, $mod) = @_;
+my @rv;
 if ($ver =~ /^(1)\.(3)(\d+)$/) {
 	$ver = sprintf "%d.%d%2.2d", $1, $2, $3;
 	}
-foreach $d (@$aref) {
+foreach my $d (@$aref) {
 	local(%dir);
 	$dir{'name'} = $d->[0];
 	$dir{'multiple'} = $d->[1];
 	$dir{'type'} = int($d->[2]);
 	$dir{'subtype'} = $d->[2] - $dir{'type'};
-	$dir{'module'} = $_[2];
+	$dir{'module'} = $mod;
 	$dir{'version'} = $ver;
 	$dir{'priority'} = $d->[5];
 	foreach $c (split(/\s+/, $d->[3])) { $dir{$c}++; }
@@ -1457,10 +1469,11 @@ foreach $f (@main::locked_apache_files) {
 }
 
 # directive_lines(directive, ...)
+# Convery a list of Apache directives into a list of lines
 sub directive_lines
 {
-local @rv;
-foreach $d (@_) {
+my @rv;
+foreach my $d (@_) {
 	next if ($d->{'name'} eq 'dummy');
 	my $indent = (" " x $d->{'indent'});
 	if ($d->{'type'}) {
@@ -1825,46 +1838,46 @@ for(my $i=0; $i<$secs; $i++) {
 return 0;
 }
 
-# configurable_modules()
+# configurable_modules([&all-mods])
 # Returns a list of Apaches that are compiled in or dynamically loaded, and
 # supported by Webmin.
 sub configurable_modules
 {
-local ($ver, $mods) = &httpd_info(&find_httpd());
-local @rv;
-local $m;
+my ($allmods) = @_;
+$allmods ||= [ &available_modules() ];
+return grep { -r "$module_root_directory/$_.pl" } @$allmods;
+}
+
+# available_modules()
+# Returns a list of Apaches that are compiled in or dynamically loaded
+sub available_modules
+{
+my ($ver, $mods) = &httpd_info(&find_httpd());
+my @rv;
 
 # Add compiled-in modules
-foreach $m (@$mods) {
-	if (-r "$module_root_directory/$m.pl") {
-		push(@rv, $m);
-		}
-	}
+push(@rv, @$mods);
 
 # Add dynamically loaded modules
-local $conf = &get_config();
-foreach $l (&find_directive_struct("LoadModule", $conf)) {
-	if ($l->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/ &&
-	    -r "$module_root_directory/$1.pl") {
+my $conf = &get_config();
+foreach my $l (&find_directive_struct("LoadModule", $conf)) {
+	if ($l->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/) {
 		push(@rv, $1);
 		}
-	elsif ($l->{'words'}->[1] =~ /libssl\.so/ &&
-	       -r "$module_root_directory/mod_apachessl.pl") {
+	elsif ($l->{'words'}->[1] =~ /libssl\.so/) {
 		push(@rv, "mod_apachessl");
 		}
-	elsif ($l->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/ &&
-	       -r "$module_root_directory/mod_$1.pl") {
+	elsif ($l->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/) {
 		push(@rv, "mod_$1");
 		}
 	}
-undef(@get_config_cache);	# Cache is no longer valid
 
 # Add dynamically loaded modules
 if ($config{'apachectl_path'}) {
 	&open_execute_command(APACHE,
 		"$config{'apachectl_path'} -M 2>/dev/null", 1);
 	while(<APACHE>) {
-		if (/(\S+)_module/ && -r "$module_root_directory/mod_${1}.pl") {
+		if (/(\S+)_module/) {
 			push(@rv, "mod_${1}");
 			}
 		}
