@@ -4,6 +4,7 @@
 # Licensed under MIT (https://github.com/authentic-theme/authentic-theme/blob/master/LICENSE)
 #
 use strict;
+use Fcntl qw(:flock O_RDWR O_CREAT);
 
 our (%in,
      %text,
@@ -45,6 +46,28 @@ sub settings
     }
 }
 
+# decode_utf8(\$scalar , [\$err])
+# Decode UTF-8 string by reference. Optionally, return error message
+sub decode_utf8 {
+    my ($sref, $errref) = @_;
+    return 0 unless defined($sref) && ref($sref) eq 'SCALAR';
+    my $tmp = $$sref;
+    my $ok  = eval {
+        require utf8;
+        utf8::decode($tmp);
+        1;
+    };
+
+    if ($ok) {
+        $$sref = $tmp;
+        return 1;
+        }
+    else {
+        $$errref = $@ || 'utf8 decode failed' if ($errref);
+        return 0;
+        }
+}
+
 sub ui_button_group_local
 {
     my ($buttons, $extra_class) = @_;
@@ -53,123 +76,6 @@ sub ui_button_group_local
     return $rv;
 }
 
-sub ui_element
-{
-    my ($e, $c) = @_;
-
-    # Allowed elements to use 'type' attr
-    if ($e =~ /^(a|button|embed|input|link|menu|object|script|source|style)$/) {
-
-        # Set default `type` for common elements
-        if ($e eq 'input') {
-            $c->{'type'} = "text" if (!$c->{'type'});
-        } elsif ($e eq 'button') {
-            $c->{'type'} = "button" if (!$c->{'type'});
-        }
-        $c->{'type'} = " type=\"@{[quote_escape($c->{'type'})]}\"" if ($c->{'type'});
-    } else {
-        delete $c->{'type'};
-    }
-
-    # Allowed elements to use 'name' attr
-    if ($e =~ /^(button|fieldset|form|iframe|input|map|meta|object|output|param|select|textarea)$/) {
-        $c->{'name'} = " name=\"@{[quote_escape($c->{'name'})]}\"" if ($c->{'name'});
-    } else {
-        delete $c->{'name'};
-    }
-
-    # Allowed elements to use 'value' attr
-    if ($e =~ /^(button|input|meter|li|option|progress|param)$/) {
-        $c->{'value'} = " value=\"@{[quote_escape($c->{'value'})]}\"" if ($c->{'value'});
-    } else {
-        delete $c->{'value'};
-    }
-
-    # Trigger `autofocus` or remove it
-    if ($e =~ /^(button|input|select|textarea)$/) {
-        if ($c->{'autofocus'} =~ /^(false|0)$/) {
-            delete $c->{'autofocus'};
-        }
-
-        # If element is empty trigger autofocus
-        elsif ($e =~ /input|textarea/ && !$c->{'value'}) {
-            $c->{'autofocus'} = " autofocus";
-        }
-    }
-
-    # Set default `class` for common elements
-    if (!$c->{'class'}) {
-        my $c_;
-        if ($e eq 'input') {
-            $c_ = "form-control";
-            if ($c->{'type'} =~ /password/) {
-                $c_ .= " ui_password";
-            } else {
-                $c_ .= " ui_textbox";
-            }
-        } elsif ($e eq 'textarea') {
-            $c_ = "form-control ui_textarea";
-        }
-        $c->{'class'} = " class=\"$c_\"";
-    } else {
-        $c->{'class'} = " class=\"@{[quote_escape($c->{'class'})]}\"";
-    }
-
-    # Collect all other independent attributes
-    my $attrs;
-    foreach my $attr (keys %{$c}) {
-        if ($attr ne "class" &&
-            $attr ne "value" &&
-            $attr ne "name"  &&
-            $attr ne "type")
-        {
-
-            # Parse and add data attributes passed as reference
-            if ($attr eq "data") {
-                if (ref($c->{$attr})) {
-                    foreach my $dattr (keys %{ $c->{$attr} }) {
-                        $attrs .= " data-$dattr=\"@{[quote_escape($c->{$attr}->{$dattr})]}\"";
-                    }
-                }
-
-                # Add all other attributes to the tag
-            } else {
-                $attrs .= " $attr=\"@{[quote_escape($c->{$attr})]}\"";
-            }
-        }
-    }
-
-    # Check if a tag must be closed
-    my $e_c;
-    if ($e !~ /^(area|base|br|hr|input|link|meta|param|source|circle|track|wbr)$/) {
-        $e_c = "</$e>";
-    }
-    return "<$e$c->{'type'}$c->{'name'}$c->{'value'}$c->{'class'}$attrs>@{[html_escape($c->{'_'})]}$e_c\n";
-}
-
-sub ui_input
-{
-    my ($c, $v, $s, $d, $m, $t) = @_;
-
-    # If old type input used, support it as well
-    if (!ref($c)) {
-        $c = { 'name'  => $c,
-               'value' => $v,
-               'size'  => $s };
-        $c->{'disabled'}  = "true" if ($d);
-        $c->{'maxlength'} = $m     if ($m);
-        if ($t) {
-            my @t = split(/\s+/, $t);
-            foreach my $t (@t) {
-                my ($t, $v) = ($t =~ /(.*?)=(.*)/);
-                $v =~ s/^("|')|("|')$//g;
-                $c->{$t} = "@{[quote_escape($v)]}";
-            }
-        }
-    }
-    return ui_element('input', $c);
-}
-
 sub ui_span_local
 {
     my ($data, $extra_class) = @_;
@@ -219,6 +125,7 @@ sub theme_ui_checkbox_local
         $label = $1;
         $after = $2;
     }
+    $after = "<span class=\"awobject-after\">$after</span>" if ($after && $after !~ /^\s*<a/);
     $label = trim($label);
     my $bl = string_ends_with($label, '<br>') ? ' ds-bl-fs' : undef;
     return "<span class=\"awcheckbox awobject$bl$cls\"><input class=\"iawobject\" type=\"checkbox\" " .
@@ -348,6 +255,16 @@ sub get_theme_language
 
 }
 
+sub get_theme_language_login
+{
+    my %s;
+    foreach my $key (keys %theme_text) {
+        next if ($key !~ /^session_/);
+        $s{$key} = $theme_text{$key};
+    }
+    return convert_to_json(\%s);
+}
+
 sub get_user_allowed_gpg_keys
 {
     my ($switch_to_user, $list_avoided_system_keys, $self) = @_;
@@ -545,7 +462,7 @@ sub replace_meta
 
 sub product_version_update_remote
 {
-    my ($latest_known_versions_remote, $latest_known_versions_remote_error, %versions_remote);
+    my ($latest_known_versions_remote, $latest_known_versions_remote_error, %versions_available);
     my $software_latest_cache       = theme_cached('software+latest');
     my $software_latest_cache_extra = sub {
         my ($software_latest_cache_original) = @_;
@@ -566,18 +483,30 @@ sub product_version_update_remote
     if ($software_latest_cache) {
         return &$software_latest_cache_extra($software_latest_cache);
     } else {
-        return { 'no-cache' => 1 } if (!post_has('xhr-'));
-        http_download("virtualmin.com", 443, '/software+latest',
-                      \$latest_known_versions_remote,
-                      \$latest_known_versions_remote_error,
-                      undef, 1, undef, undef, 10);
-        if ($latest_known_versions_remote &&
-            !$latest_known_versions_remote_error)
-        {
-            %versions_remote = map {split /=/, $_} (split(/\n/, $latest_known_versions_remote));
+        my $packages_updates_mod = 'package-updates';
+        my $packages_updates     = &foreign_available($packages_updates_mod);
+        return { 'no-cache' => 1 }
+          if (!post_has('xhr-') ||
+              !$packages_updates);
+        if ($packages_updates) {
+            &foreign_require($packages_updates_mod);
+            my @packages_updates_current = &package_updates::list_for_mode('updates', 0);
+            if (@packages_updates_current) {
+                foreach my $package_current (@packages_updates_current) {
+                    my ($package, $version) = ($package_current->{'name'}, $package_current->{'version'});
+                    if ($package &&
+                        $version &&
+                        $package =~ /^(wbm|wbt|ust|webmin|usermin)/)
+                    {
+                        $package =~ s/^(wbm|wbt|ust|webmin|usermin)\-//;
+                        $versions_available{$package} = $version;
+                    }
+
+                }
+            }
         }
-        theme_cached('software+latest', \%versions_remote, $latest_known_versions_remote_error);
-        return &$software_latest_cache_extra(\%versions_remote);
+        theme_cached('software+latest', \%versions_available);
+        return &$software_latest_cache_extra(\%versions_available);
     }
 
 }
@@ -600,25 +529,49 @@ sub product_version_update
       $product_local_name eq "f" ? ["ConfigServer Security & Firewall", $software_versions_remote->{'csf'}] :
       "";
 
+    # A work-around to fix inconsistency returned by `module.info` (i.e. 7.3.gpl)
+    # and what package manager provides (i.e. 7.3.gpl-1).
+    if ($product_local_name =~ /^(w|u|v|c)$/) {
+        $product_local_version =~ s/(\d+\.\d+\.\d+|\d+\.\d+)(.*)/$1/
+          if ($product_local_version);
+        $software_versions_remote->{'webmin'} =~ s/(\d+\.\d+\.\d+|\d+\.\d+)(.*)/$1/
+          if ($software_versions_remote->{'webmin'});
+        $software_versions_remote->{'usermin'} =~ s/(\d+\.\d+\.\d+|\d+\.\d+)(.*)/$1/
+          if ($software_versions_remote->{'usermin'});
+        $software_versions_remote->{'virtual-server'} =~ s/(\d+\.\d+\.\d+|\d+\.\d+)(.*)/$1/
+          if ($software_versions_remote->{'virtual-server'});
+        $software_versions_remote->{'server-manager'} =~ s/(\d+\.\d+\.\d+|\d+\.\d+)(.*)/$1/
+          if ($software_versions_remote->{'server-manager'});
+    }
+
     # Compare versions
     if (
         ($product_local_name eq "w" &&
-         &compare_version_numbers($product_local_version, $software_versions_remote->{'webmin'}) < 0
-        ) ||
+         $product_local_version &&
+         &compare_version_numbers($product_local_version, $software_versions_remote->{'webmin'}) < 0) ||
         ($product_local_name eq "u" &&
+            $product_local_version &&
             &compare_version_numbers($product_local_version, $software_versions_remote->{'usermin'}) < 0) ||
         ($product_local_name eq "v" &&
+            $product_local_version &&
             &compare_version_numbers($product_local_version, $software_versions_remote->{'virtual-server'}) < 0) ||
         ($product_local_name eq "c" &&
+            $product_local_version &&
             &compare_version_numbers($product_local_version, $software_versions_remote->{'server-manager'}) < 0) ||
         ($product_local_name eq "f" &&
-            &compare_version_numbers($product_local_version, $software_versions_remote->{'csf'}) < 0))
-    {
-        return '<a href="https://forum.virtualmin.com/search?q=' .
-          $product_remote_version->[0] . '%20in%3Atitle%20%23news%20order%3Alatest" target="_blank">' .
-          '<span data-toggle="tooltip" data-placement="auto top" data-title="' .
-          theme_text('theme_xhred_global_outdated_desc', $product_remote_version->[0], $product_remote_version->[1]) .
-          '" class="bg-danger text-danger pd-lf-2 pd-rt-2 br-2">' . $product_local_version . '</span></a>';
+            $product_local_version &&
+            &compare_version_numbers($product_local_version, $software_versions_remote->{'csf'}) < 0)) {
+        if (&foreign_available("virtual-server")) {
+            return '<a href="https://forum.virtualmin.com/search?q=' .
+              $product_remote_version->[0] . '%20in%3Atitle%20%23news%20order%3Alatest" target="_blank">' .
+              '<span data-toggle="tooltip" data-placement="auto top" data-title="' .
+              theme_text('theme_xhred_global_outdated_desc', $product_remote_version->[0], $product_remote_version->[1]) .
+              '" class="bg-danger text-danger pd-lf-2 pd-rt-2 br-2">' . $product_local_version . '</span></a>';
+        } else {
+            return '<span data-toggle="tooltip" data-placement="auto top" data-title="' .
+              theme_text('theme_xhred_global_outdated_desc2', $product_remote_version->[0], $product_remote_version->[1]) .
+              '" class="bg-danger text-danger pd-lf-2 pd-rt-2 br-2">' . $product_local_version . '</span>';
+        }
     } else {
         return $product_local_version;
     }
@@ -776,57 +729,6 @@ sub current_to_pid
     write_file($tmp_file, \%pid);
 }
 
-sub network_stats
-{
-    # Get network data from all interfaces
-    my ($type) = @_;
-    my $file = "/proc/net/dev";
-    return () unless -r $file;
-    open(my $dev, $file);
-    my (@titles, %result);
-    while (my $line = <$dev>) {
-        chomp($line);
-        if ($line =~ /^.{6}\|([^\\]+)\|([^\\]+)$/) {
-            my ($rec, $trans) = ($1, $2);
-            @titles = ((map {"r$_"} split(/\s+/, $rec)), (map {"t$_"} split(/\s+/, $trans)));
-        } elsif ($line =~ /^\s*([^:]+):\s*(.*)$/) {
-            my ($id, @data) = ($1, split(/\s+/, $2));
-            $result{$id} = { map {$titles[$_] => $data[$_];} (0 .. $#titles) };
-        }
-    }
-    close($dev);
-
-    # Return current network I/O
-    if ($type eq 'io') {
-        my ($rbytes, $tbytes, $rbytes2, $tbytes2) = (0, 0, 0, 0);
-        my @rs;
-        my $results = \%result;
-
-        # Parse current data
-        foreach (%$results) {
-            $rbytes += $results->{$_}->{'rbytes'};
-            $tbytes += $results->{$_}->{'tbytes'};
-        }
-
-        # Wait for one second and fetch data over again
-        sleep 1, $results = network_stats();
-
-        # Parse data after dalay
-        foreach (%$results) {
-            $rbytes2 += $results->{$_}->{'rbytes'};
-            $tbytes2 += $results->{$_}->{'tbytes'};
-        }
-
-        # Return current network I/O
-        $rbytes = int($rbytes2 - $rbytes);
-        $tbytes = int($tbytes2 - $tbytes);
-
-        @rs = ($rbytes, $tbytes);
-        return serialise_variable(\@rs);
-    }
-    return \%result;
-}
-
 sub acl_system_status
 {
     my ($show) = @_;
@@ -861,7 +763,24 @@ sub get_default_module
 # Webmin manually. Inner ref.: CXX1010000
 sub init_prefail
 {
+    # Usermin is always fully restarted
+    return if (get_product_name() eq 'usermin');
+    my $prod_version = get_webmin_version();
     if (
+        # Affects upgrades 2.120 and up
+        (!defined(&get_miniserv_websocket_url) && $prod_version >= 2.120) ||
+        
+        # Affects upgrades 2.103 and up
+        (!defined(&ui_switch_theme_javascript) && $prod_version >= 2.103) ||
+        
+        # Affects upgrades 2.022 and up
+        (!defined(&get_http_redirect) && $prod_version >= 2.022) ||
+        (!defined(&create_wrapper) && $prod_version >= 2.022) ||
+
+        # Affects upgrades before 2.020
+        !defined(&parse_accepted_language) ||
+        !defined(&get_default_system_locale) ||
+
         # Affects upgrades before 1.974
         !defined(&get_buffer_size) ||
 
@@ -873,13 +792,19 @@ sub init_prefail
         !defined(&setvar) ||
 
         # Affects upgrades from before 1.995
-        !defined(&webmin_user_can_rpc) ||
-        !defined(&webmin_user_is_admin))
+        !defined(&webmin_user_can_rpc)  ||
+        !defined(&webmin_user_is_admin) ||
+
+        # Affects upgrades from before 2.000
+        !defined(&get_webmin_full_version) ||
+
+        # Affects upgrades from before 2.002
+        !defined(&miniserv_using_default_cert))
     {
         load_theme_library();
         do("$root_directory/web-lib-funcs.pl");
         setvar('needs-restart', has_command('systemctl') || $config_directory)
-            if (defined(&setvar));
+          if (defined(&setvar));
     }
 }
 
@@ -894,4 +819,98 @@ sub post_has
     return 0;
 }
 
+# Read file contents
+sub theme_read_file_contents {
+    my ($filename) = @_;
+    # Check if file is readable
+    return undef unless -r $filename;
+    # Read file contents
+    my $fh;
+    if (!open($fh, '<', $filename)) {
+        error_stderr("Failed to open file '$filename' for reading: $!");
+        return undef;
+    }
+    # Acquire lock
+    if (!flock($fh, LOCK_SH)) {
+        error_stderr("Failed to acquire shared lock on file '$filename': $!");
+        close($fh);
+        undef($fh);
+        return undef;
+    }
+    # Read file contents
+    local $/;
+    my $contents = <$fh>;
+    # Release lock
+    flock($fh, LOCK_UN);
+    # Close file
+    if (!close($fh)) {
+        error_stderr("Failed to close file '$filename': $!");
+        undef($fh);
+        return undef;
+    }
+    # Release memory
+    undef($fh);
+    # Return file contents
+    return $contents;
+}
+
+# Write file contents
+sub theme_write_file_contents {
+    my ($filename, $contents) = @_;
+    my $fh;
+    my $cleanup = sub {
+        my ($unlock, $success) = @_;
+        # Unlock the file. Not strictly necessary as Perl
+        # automatically releases locks on file closure
+        flock($fh, LOCK_UN) if ($unlock);
+        # Close file handle
+        close($fh)
+            or do {
+                error_stderr("Failed to close file '$filename': $!");
+                undef($fh);
+                return 0;
+            };
+        # Release memory. Generally not necessary as Perl's garbage collection
+        # should always handle this. However, it doesn't hurt and can be
+        # useful in certain scenarios, like long-running processes
+        undef($fh);
+        # Return result
+        return $success ? 1 : 0;
+    };
+    # Open file for reading and writing without truncating it, or 
+    # create it if it doesn't exist
+    sysopen($fh, $filename, O_RDWR | O_CREAT)
+        or do {
+            error_stderr("Failed to open file '$filename' for reading and writing: $!");
+            return 0;
+        };
+    # Acquire exclusive lock on the file for writing
+    flock($fh, LOCK_EX)
+        or do {
+            error_stderr("Failed to acquire exclusive lock on file '$filename': $!");
+            return $cleanup->(0);
+        };
+    # Truncate the file after acquiring the lock
+    truncate($fh, 0)
+        or do {
+            error_stderr("Failed to truncate file '$filename': $!");
+            return $cleanup->(1);
+        };
+    # Reset the file pointer to the beginning of the file
+    # preventing potential race conditions
+    seek($fh, 0, 0)
+        or do {
+            error_stderr("Failed to seek to the beginning of file '$filename': $!");
+            return $cleanup->(1);
+        };
+    # Write to file
+    print {$fh} $contents
+        or do {
+            error_stderr("Failed to write to file '$filename': $!");
+            return $cleanup->(1);
+        };
+    # Clean up and return success
+    return $cleanup->(1, 1);
+}
+
 1;
