@@ -39,6 +39,7 @@ our (@theme_bundle_css,
      $title);
 
 do($ENV{'THEME_ROOT'} . "/authentic-funcs.pl");
+
 init_prefail();
 init_vars();
 
@@ -77,22 +78,28 @@ sub settings_filter
 sub get_theme_color
 {
     # Chrome application header color should be darker than the theme color
-    my %theme_colors = ('blue'   => '#003670', # '#004090',
-                        'teal'   => '#02474c', # '#03535c',
-                        'green'  => '#234b2d', # '#2a5c36',
-                        'purple' => '#45345e', # '#533e71',
-                        'brown'  => '#4b3a2e', # '#594739',
+    my %theme_colors = ('blue'   => '#003670', # '#033e86',
+                        'teal'   => '#02474c', # '#034b53',
+                        'green'  => '#234b2d', # '#2b5936',
+                        'purple' => '#45345e', # '#4f3a6e',
+                        'brown'  => '#4b3a2e', # '#544336',
                         'gold'   => '#61482e', # '#74573a',
-                        'orange' => '#724d2a', # '#885c34',
+                        'orange' => '#724d2a', # '#7e5631',
                         'red'    => '#7e2b27', # '#9a3531',
                         'maroon' => '#6f233e', # '#862b4c',
-                        'grey'   => '#2f3336', # '#3a4043',
+                        'grey'   => '#2f3336', # '#343c42',
                         'white'  => '#555555');# '#ffffff'
 
     my $color = $theme_config{'settings_navigation_color'};
     return $theme_colors{$color};
 }
 
+sub load_devel_dependencies
+{
+do("$ENV{'THEME_ROOT'}/dependencies.pl")
+    if (!@theme_bundle_css || !@theme_bundle_js);
+}
+
 sub embed_favicon
 {
     my ($is_login_page) = @_;
@@ -109,7 +116,7 @@ sub embed_favicon
     }
 
     my $theme_config_dir = "$config_directory/$current_theme";
-    my $theme_user_color = get_theme_color() || "#004090";
+    my $theme_user_color = get_theme_color() || "#033e86";
 
     my $favicon_path = $theme_webprefix . '/images/favicons/' . $product_name;
     my $ref_link     = 'data-link-ref';
@@ -182,9 +189,9 @@ sub embed_favicon
 
     # Generate manifest file from template
     my $display_hostname         = get_display_hostname();
-    my $manifest_product_name_uc = ucfirst($product_name);
-    my $manifest_product_name_uc_with_hostname =
-        $manifest_product_name_uc . ($display_hostname ? " on " . $display_hostname : "");
+    my $manifest_product_name_uc = $theme_config{'settings_manifest_product_name'} || ucfirst($product_name);
+    my $manifest_product_name_uc_with_hostname = $theme_config{'settings_manifest_product_name_full'} ||
+        ($manifest_product_name_uc . ($display_hostname ? " on " . $display_hostname : ""));
     my %manifest_prod_descs = ('webmin'     => 'Powerful and flexible web-based server management control panel',
                                 'usermin'    => 'Powerful and flexible web-based user management interface',
                                 'virtualmin' => 'Powerful and flexible web hosting control panel',
@@ -305,22 +312,13 @@ sub embed_header
     print "</script>\n";
 
     if ($args[2]) {
-        do($ENV{'THEME_ROOT'} . "/dependencies.pl");
+        load_devel_dependencies();
     }
 
     if ($args[3] eq '1') {
 
         if ($args[2]) {
-            foreach my $css (@theme_bundle_css) {
-                print ' <link href="' .
-                  $theme_webprefix .
-                  '/unauthenticated/css/' .
-                  $css .
-                  '.src.css?' .
-                  theme_version('timestamped') .
-                  '" rel="stylesheet">' . "\n";
-            }
-            embed_css_fonts();
+            embed_css_unbundled();
         } else {
             embed_css_bundle();
         }
@@ -412,7 +410,7 @@ sub embed_header
     
     # Embed theme switcher
     print &ui_switch_theme_javascript()
-        if (defined(&ui_switch_theme_javascript));
+        if ($main::session_id && defined(&ui_switch_theme_javascript));
     
     print '</head>', "\n";
 }
@@ -580,6 +578,18 @@ sub embed_css_fonts
     }
 }
 
+sub embed_css_unbundled
+{
+    load_devel_dependencies();
+    foreach my $css (@theme_bundle_css) {
+        printf " <link href=\"%s/unauthenticated/css/%s.src.css?%s\" rel=\"stylesheet\">\n",
+               $theme_webprefix,
+               $css,
+               theme_version('timestamped');
+    }
+    embed_css_fonts();
+}
+
 sub embed_css_bundle
 {
     print ' <link href="' .
@@ -1029,6 +1039,27 @@ sub get_button_style
         if (string_contains($keys, 'ssl_gen')) {
             $class = "grey ";
         }
+    } elsif (string_contains($keys, "licence_manager") ||
+             string_contains($keys, "licence_manager_change") ||
+             string_contains($keys, "licence_manager_deregister") ||
+             string_contains($keys, "licence_recheck")) {
+        $icon = "virtualmin fa-1_15x margined-left--2 margined-top-1";
+        $class = "info ";
+        if (string_contains($keys, "licence_recheck")) {
+            $icon = "refresh fa-1_05x margined-left--2";
+        } elsif (string_contains($keys, "licence_manager_goto")) {
+            $class = "warning ";
+            $icon = " fa2 fa2-key fa-1_15x margined-left--1";
+        }
+        if (string_contains($keys, "licence_manager_change") ||
+            string_contains($keys, "licence_manager_setup")) {
+            $class = "success ";
+            $icon = "unlock fa-1_15x margined-left--2";
+        }
+        if (string_contains($keys, "licence_manager_deregister")) {
+            $class = "grey ";
+            $icon = "lock fa-1_15x margined-left--2";
+        }
     } elsif (string_contains($keys, "user_createover")) {
         $class = "warning ";
         $icon  = "user-switch fa-1_10x";
@@ -1052,9 +1083,11 @@ sub get_button_style
     } elsif (string_contains($keys, "mail_delete")) {
         $icon  = "times-circle";
         $class = "danger ";
-    } elsif (string_contains($keys, "mail_forward")) {
+    } elsif (string_contains($keys, "mail_forward") ||
+             string_contains($keys, "view_forward")) {
         $icon = "forward";
-    } elsif (string_contains($keys, "view_quick_all")) {
+    } elsif (string_contains($keys, "view_quick_all") ||
+             string_contains($keys, "view_reply2")) { 
         $icon = "reply-all";
     } elsif (string_contains($keys, "view_reply")) {
         $icon  = "reply";
@@ -1109,13 +1142,13 @@ sub get_button_style
         $icon  = "check-circle-o";
     } elsif (string_contains($keys, "migrate_show") ||
              string_contains($keys, "import_show") ||
-             string_contains($keys, "keys_import"))
-    {
+             string_contains($keys, "keys_import")) {
         $class = "success ";
         $icon  = " fa2 fa2-import";
+    } elsif (string_contains($keys, "index_refresh")) {
+        $icon  = "refresh-mdi fa-1_25x";
     } elsif (string_contains($keys, "update") ||
-             string_contains($keys, "index_sync"))
-    {
+             string_contains($keys, "index_sync")) {
         $class = "info ";
         $icon  = "refresh";
     } elsif ((string_contains($keys, "delete") && !string_contains($keys, "users_delete")) ||
@@ -1154,9 +1187,11 @@ sub get_button_style
     } elsif (string_contains($keys, "twofactor_enable")) {
         $class = "info ";
         $icon  = "lock";
-    } elsif (string_contains($keys, "blockip")) {
+    } elsif (string_contains($keys, "allowip") || string_contains($keys, "blockip")) {
+        my $icon_type = string_contains($keys, "allowip") ?
+                                        "unlock" : " fa2 fa2-not-interested";
         $class = "grey ";
-        $icon  = " fa2 fa2-not-interested fa-1_10x margined-left--2";
+        $icon  = "$icon_type fa-1_10x margined-left--2";
     } elsif (string_contains($keys, "twofactor_disable")) {
         $class = "warning ";
         $icon  = "unlock";
@@ -1168,6 +1203,9 @@ sub get_button_style
         } else {
             $icon = " fa2 fa2-key";
         }
+    } elsif (string_contains($keys, "mods_install") ||
+             string_contains($keys, "pkgs_install")) {
+        $icon  = "package-install fa-1_25x";
     } elsif (
              (string_contains($keys, "install")     ||
               string_contains($keys, "recsok")      ||
@@ -1309,8 +1347,7 @@ sub get_button_style
         $class = "warning ";
         $icon  = " fa2 fa2-transfer";
     } elsif (string_contains($keys, "restart") ||
-             string_contains($keys, "edit_kill") ||
-             string_contains($keys, "licence_recheck"))
+             string_contains($keys, "edit_kill"))
     {
         $class = "warning ";
         $icon  = "refresh";
@@ -1409,6 +1446,11 @@ sub get_button_style
     } elsif (string_contains($keys, "databases_import")) {
         $class = "success ";
         $icon  = "database-plus fa-1_25x";
+    } elsif (
+         string_contains($keys, "index_fadd")
+         )
+    {
+        $icon  = "plus-square-o";
     } elsif (
          (string_contains($keys, "add") &&
           !string_contains($keys, "edit_addinc")) ||
@@ -1543,11 +1585,17 @@ sub embed_login_head
     print '<head>',                                           "\n";
     print ' <meta name="color-scheme" content="only light">', "\n";
     embed_noscript();
-    print '<meta charset="utf-8">', "\n";
+    print ' <meta charset="utf-8">', "\n";
+    print ' <script type="text/javascript">try{const e=new URL(location.href),a=e.pathname,s=/\/(session_login|pam_login)\.cgi$/.test(a),r=e.searchParams.has("logout"),t=e.searchParams.has("returned-username")&&/^\S+$/.test(e.searchParams.get("returned-username"));(s&&r||t)&&(e.pathname=a.substring(0,a.lastIndexOf("/")),e.searchParams.delete("logout"),e.searchParams.delete("returned-username"),history.pushState({},"",e))}catch(e){}</script>', "\n";
     embed_favicon('login-page');
-    print '<title>', $title, '</title>', "\n";
-    print '<meta name="viewport" content="width=device-width, initial-scale=1.0">' . "\n";
-
+    print ' <title>', $title, '</title>', "\n";
+    print ' <meta name="viewport" content="width=device-width, initial-scale=1.0">' . "\n";
+    # Print object with language strings
+    if ($gconfig{'forgot_pass'}) {
+        print ' <script>';
+        print 'const theme_language = ' . get_theme_language_login();
+        print "</script>\n";
+    }
     if ($inline) {
         my $file_contents = read_file_contents("$root_directory/$current_theme/unauthenticated/css/bundle.min.css");
         print '<style>';
@@ -1567,16 +1615,19 @@ sub embed_login_head
         print '</style>';
 
     } else {
-        print '<link href="' . $theme_webprefix .
-          '/unauthenticated/css/bundle.min.css?' . theme_version('timestamped') . '" rel="stylesheet">' . "\n";
-
-        print
-'<script>document.addEventListener("DOMContentLoaded", function(event) {var a=document.querySelectorAll(\'input[type="password"]\');i=0;
-for(length=a.length;i<length;i++){var b=document.createElement("span"),d=30<a[i].offsetHeight?1:0;b.classList.add("input_warning_caps");b.setAttribute("title","Caps Lock");d&&b.classList.add("large");a[i].classList.add("use_input_warning_caps");a[i].parentNode.insertBefore(b,a[i].nextSibling);a[i].addEventListener("blur",function(){this.nextSibling.classList.remove("visible")});a[i].addEventListener("keydown",function(c){"function"===typeof c.getModifierState&&((state=20===c.keyCode?!c.getModifierState("CapsLock"):
-c.getModifierState("CapsLock"))?this.nextSibling.classList.add("visible"):this.nextSibling.classList.remove("visible"))})};});function spinner() {var x = document.querySelector(\'button i.fa-sign-in:not(.invisible)\') || document.querySelector(\'button i.fa-qrcode:not(.invisible)\'),s = \'<span class="cspinner_container"><span class="cspinner"><span class="cspinner-icon white small"></span></span></span>\';if(x){x.classList.add("invisible"); x.insertAdjacentHTML(\'afterend\', s);x.parentNode.classList.add("disabled");x.parentNode.disabled=true}}setTimeout(function(){if(navigator&&navigator.oscpu){var t=navigator.oscpu,i=document.querySelector("html"),e="data-platform";t.indexOf("Linux")>-1?i.setAttribute(e,"linux"):t.indexOf("Windows")>-1&&i.setAttribute(e,"windows")}});</script>';
+        if ($ext eq 'src') {
+            embed_css_unbundled();
+        } else {
+            embed_css_bundle();
+        }
 
+        print ' <script src="' .
+              $theme_webprefix .
+              "/unauthenticated/js/session-login.$ext.js?" .
+              theme_version('timestamped') .
+              '"></script>' . "\n";
+        
         embed_css_night_rider();
-        embed_css_fonts();
     }
 
     embed_background();
