@@ -21,7 +21,7 @@ our (
     $current_theme, $root_directory, $config_directory, $var_directory,
 
     %theme_text,     %module_text_full, %theme_config, $theme_info, $get_user_level, $theme_webprefix, $http_x_url,
-    $has_virtualmin, $has_cloudmin,
+    $has_virtualmin, $has_cloudmin, $theme_server_webprefix,
     $has_usermin,    $has_usermin_version, $has_usermin_root_dir, $has_usermin_conf_dir, $has_usermin_var_dir);
 
 init_type();
@@ -206,6 +206,7 @@ sub get_extended_sysinfo
                         ))
                     {
                         foreach my $t (@{ $info->{'table'} }) {
+                            $t->{"desc"} =~ s/^\&nbsp;//g;
                             $t->{"desc"} =~ s/<img src='.*?\/virtual-server\/images\/(\S+).(png|gif)'.*?>/$status_icons->{$1}/g;
                             $t->{"value"} =~ s/<img src='.*?\/virtual-server\/images\/(\S+).(png|gif)'.*?>/$status_icons->{$1}/g;
                             $returned_sysinfo .= '<tr>
@@ -238,6 +239,7 @@ sub get_extended_sysinfo
         if (&webmin_user_is_admin() &&
             $theme_config{'settings_sysinfo_hidden_panels_user'} !~ /\'live_stats\'/ &&
             $theme_config{'settings_sysinfo_real_time_status'} ne '0'     &&
+            (!$theme_server_webprefix || $theme_server_webprefix && -r get_stats_history_file()) &&
             (acl_system_status('cpu') || acl_system_status('mem') || acl_system_status('load')))
         {
             my $data = '<div data-charts-loader class="text-muted loading-dots flex-center">
@@ -252,11 +254,18 @@ sub get_extended_sysinfo
                             <span data-chart="proc"></span>
                             <span data-chart="disk"></span>
                             <span data-chart="net"></span>';
+            my $live_stats_not_live; 
+            $live_stats_not_live =
+                '<span data-live="'.
+                        ($theme_server_webprefix ? 0 : 1).'"'.get_button_tooltip(
+                                theme_text('theme_xhred_tooltip_dashboard_live_stats_offline',
+                                undef), undef, 'auto right').
+                                    '></span>' if ($theme_server_webprefix);
             $returned_sysinfo .=
               print_panel(
                 1,
                 'live_stats',
-"$theme_text{'theme_dashboard_accordion_live_stats'}<span class=\"pull-right on-hover\"><i class=\"fa fa-fw fa-times-thin\" "
+"$theme_text{'theme_dashboard_accordion_live_stats'}${live_stats_not_live}<span class=\"pull-right on-hover\"><i class=\"fa fa-fw fa-times-thin\" "
                   .
                   get_button_tooltip(
                                      theme_text('theme_xhred_tooltip_dashboard_panels_disable',
@@ -510,7 +519,7 @@ sub print_sysstats_panel_start
 
 sub print_sysstats_table
 {
-    my ($data, $quota, $prod) = @_;
+    my ($data, $quota) = @_;
 
     if ((defined($data) && scalar(@{$data})) ||
         (defined($quota) && scalar(@{$quota})))
@@ -518,14 +527,7 @@ sub print_sysstats_table
         print '<table class="table table-hover">' . "\n";
         if (defined($data) && scalar(@{$data})) {
             foreach my $t (@{ @{$data}[0]->{'table'} }) {
-                my $insert = ($t->{"desc"} =~ /\Q$prod/i);
-                if ($insert && $get_user_level ne '3') {
-                    print_table_row($theme_text{'body_webmin'}, get_webmin_version());
-                }
                 print_table_row($t->{"desc"}, $t->{"value"});
-                if ($insert) {
-                    print_table_row($theme_text{'theme_version'}, get_theme_user_link());
-                }
             }
         }
         if (defined($quota) && scalar(@{$quota})) {
@@ -901,8 +903,8 @@ sub get_sysinfo_vars
                   ($cpucores > 1 ? ($theme_text{'theme_global_core'} . ' ' . (int($t->{'core'}) + 1) . ': ') : '')
                   .
                   ( get_module_config_data('system-status', 'collect_units') ?
-                      (int(($t->{'temp'} * 9.0 / 5) + 32) . "&#176;F") :
-                      (int($t->{'temp'}) . '&#176;C ')
+                      (int(($t->{'temp'} * 9.0 / 5) + 32) . " &#176;F") :
+                      (int($t->{'temp'}) . ' &#176;C ')
                   ) .
                   '</span>';
             }
@@ -933,8 +935,8 @@ sub get_sysinfo_vars
                 $hdd_temperature .= '<span class="badge-custom badge-drivestatus" data-stats="drive">' . $short . ': '
                   .
                   ( get_module_config_data('system-status', 'collect_units') ?
-                      (int(($t->{'temp'} * 9.0 / 5) + 32) . "&#176;F") :
-                      (int($t->{'temp'}) . '&#176;C ')
+                      (int(($t->{'temp'} * 9.0 / 5) + 32) . " &#176;F") :
+                      (int($t->{'temp'}) . ' &#176;C ')
                   ) .
                   $emsg . '</span>';
             }
@@ -1796,10 +1798,6 @@ sub clear_theme_cache
         }
         &virtual_server::clear_links_cache($remote_user);
 
-        # Clear license status cache
-        my $licence_status = &virtual_server::cache_file_path("licence-status");
-        unlink_file($licence_status);
-
         # Clear collected data
         my $collected_info_file = &virtual_server::cache_file_path("collected");
         unlink_file($collected_info_file);
@@ -1872,7 +1870,7 @@ sub init_type
 
 sub init
 {
-    # Don't log XHR requests
+    # Don't log XHR requests (exclude from logs)
     my %tmp_miniserv;
     get_miniserv_config(\%tmp_miniserv);
     my $nolog = quotemeta('/stats.cgi?xhr-stats=general');
