@@ -73,7 +73,8 @@ sub theme_header
         # Print default container
         print '<div class="container-fluid col-lg-10 col-lg-offset-1" data-dcontainer="1">' . "\n";
         my %this_module_info = &get_module_info(&get_module_name());
-        print '<div class="panel panel-default">' . "\n";
+        print '<div data-module-sortable="' . ($this_module_info{'sortable'} ? 1 : 0) . 
+              '" class="panel panel-default">' . "\n";
         print '<div class="panel-heading">' . "\n";
         print $tconfig{'preheader'};
         print '<div class="header">
@@ -144,7 +145,7 @@ sub theme_header
               "+2";
             print "<div data-current-module-name=\"$this_module_info{'desc'}\" id='headln2c' class=\"col-sm-4\">",
               ($ts ? "<span data-main_title>" : ""), $ttitle, ($ts ? "</span>" : "");
-            print "<br>$_[9]\n" if ($_[9]);
+            print "<br><span data-main_title_sub>$_[9]</span>\n" if ($_[9]);
             if ($theme_ui_post_header_text) {
                 print "<span data-sub_title><br>$theme_ui_post_header_text</span>";
             } else {
@@ -324,55 +325,73 @@ sub theme_ui_upload
       ($tags ? " " . $tags : "") . ">";
 }
 
-sub theme_icons_table
-{
-    my $lt = $theme_config{'settings_right_table_links_type'};
-    my $sm = $lt eq '1' ? ' small-icons' : undef;
-    my $nl = $lt eq '0' ? 1              : undef;
+sub theme_icons_table {
+    my ($links, $titles, $icons, $cols, $href, $width, $height, $befores, $afters) = @_;
+    my $lt           = $theme_config{'settings_right_table_links_type'};
+    my $sm           = $lt eq '1' ? ' small-icons' : undef;
+    my $nl           = $lt eq '0' ? 1              : undef;
+    my $icons_sorted = $theme_config{'settings_right_table_links_sorted'} eq 'true' ? 1 : 0;
+
+    # Sort links, titles, and icons
+    if ($icons_sorted) {
+        my @sorted_indices = sort { $titles->[$a] cmp $titles->[$b] } 0 .. $#$titles;
+        @$links   = @$links[@sorted_indices];
+        @$titles  = @$titles[@sorted_indices];
+        @$icons   = @$icons[@sorted_indices];
+        @$befores = @$befores[@sorted_indices];
+        @$afters  = @$afters[@sorted_indices];
+    }
 
     if ($nl) {
-
         # Print plain text links
         print "<div class=\"links-row-padded\">";
         print
 "<div class=\"row-flex row-flex-cols-sm-1 row-flex-cols-md-2 row-flex-cols-lg-3 row-flex-cols-xl-3 row-flex-cols-xxl-4 links-row\">";
-        for (my $i = 0; $i < @{ $_[0] }; $i++) {
-            my $after;
-            if ($_[8]->[$i]) {
-                $after = $_[8]->[$i];
-                $after =~
-s/ui_link/margined-left-4 btn btn-default btn-transparent-link btn-xxs btn-xxs-compact btn-hover-hide f__lnk_t_btn-el/g;
-                $after =~ s/>\((.*?)\)</>$1</g;
-            }
-            my $before;
-            $before = $_[7]->[$i];
-            print "<div class=\"col-flex-sm-6" . ($before ? " link-row-col" : " link-row-col-narrow") . "\">";
-            print "$before <a class='row-link' href='$_[0]->[$i]' " . (ref($_[4]) ? $_[4]->[$i] : $_[4]) .
-              ">" . ($before ? "" : "<i class='fa fa-fw fa-angle-right'>&nbsp;&nbsp;</i>") .
-              "<span>@{[html_strip($_[1]->[$i], ' ')]}</span></a> $after";
+
+        for my $i (0 .. $#$links) {
+            my $after  = $afters->[$i] // '';
+            my $before = $befores->[$i] // '';
+
+            # Process "after" HTML
+            $after =~ s/ui_link/margined-left-4 btn btn-default btn-transparent-link btn-xxs btn-xxs-compact btn-hover-hide f__lnk_t_btn-el/g;
+            $after =~ s/>\((.*?)\)</>$1</g;
+
+            # Print individual link
+            print "<div class=\"col-flex-sm-6"
+              . ($before ? " link-row-col" : " link-row-col-narrow")
+              . "\">";
+            print "$before <a class='row-link' href='$links->[$i]' "
+              . (ref($href) ? $href->[$i] : $href)
+              . ">"
+              . ($before ? "" : "<i class='fa fa-fw fa-angle-right'>&nbsp;&nbsp;</i>")
+              . "<span>@{[html_strip($titles->[$i], ' ')]}</span></a> $after";
             print "</div>";
         }
-        print "</div>";
-        print "</div>";
 
+        print "</div>";
+        print "</div>";
     } else {
-
         # Print icons
         my $ff;
-        my $in = scalar(@{ $_[0] });
+        my $total = scalar(@$links);
         print "<div class=\"row icons-row$sm\">\n";
-        for (my $i = 0; $i < @{ $_[0] }; $i++) {
-            &generate_icon($_[2]->[$i], $_[1]->[$i], $_[0]->[$i], ref($_[4]) ? $_[4]->[$i] : $_[4],
-                           $_[5], $_[6], $_[7]->[$i], $_[8]->[$i]);
-            $ff .= '<div class="icons-container-filler"></div>', "\n" if (!$sm);
+
+        for my $i (0 .. $#$links) {
+            &generate_icon($icons->[$i], $titles->[$i], $links->[$i],
+                ref($href) ? $href->[$i] : $href,
+                $width, $height, $befores->[$i], $afters->[$i]);
+
+            $ff .= '<div class="icons-container-filler"></div>' if (!$sm);
         }
-        if ($in < 2) {
+
+        # Adjust fillers for row balancing
+        if ($total < 2) {
             $ff .= "$ff" x 16;
-        } elsif ($in < 4) {
+        } elsif ($total < 4) {
             $ff .= "$ff" x 8;
-        } elsif ($in < 8) {
+        } elsif ($total < 8) {
             $ff .= "$ff" x 4;
-        } elsif ($in < 16) {
+        } elsif ($total < 16) {
             $ff .= "$ff" x 2;
         }
         print "$ff</div>\n";
@@ -538,11 +557,13 @@ sub theme_ui_help
 
 sub theme_hlink
 {
-    my $mod    = $_[2] ? $_[2] : &get_module_name();
-    my $width  = $_[3] || $tconfig{'help_width'}  || $gconfig{'help_width'}  || 600;
-    my $height = $_[4] || $tconfig{'help_height'} || $gconfig{'help_height'} || 400;
+    my ($txt, $page, $mod, $width, $height, $tmpl) = @_;
+    $mod ||= &get_module_name();
+    $width  = $width || $tconfig{'help_width'}  || $gconfig{'help_width'}  || 600;
+    $height = $height || $tconfig{'help_height'} || $gconfig{'help_height'} || 400;
+    my $params = $tmpl ? "?".join("&", map { "tmpl_".&urlize($_)."=".&urlize($tmpl->{$_}) } keys %{$tmpl}) : "";
     return
-"<a onClick='window.open(\"$theme_webprefix/help.cgi/$mod/$_[1]\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=$width,height=$height,resizable=yes\"); return false' href=\"$theme_webprefix/help.cgi/$mod/$_[1]\">$_[0]</a>";
+"<a onClick='window.open(\"$theme_webprefix/help.cgi/$mod/$page$params\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=$width,height=$height,resizable=yes\"); return false' href=\"$theme_webprefix/help.cgi/$mod/$page$params\">$txt</a>";
 }
 
 sub theme_ui_link
@@ -695,11 +716,13 @@ sub theme_ui_textbox
 
 sub theme_ui_password
 {
-    my ($name, $value, $size, $dis, $max, $tags) = @_;
-    my $rv;
-
-    $rv .=
-'<input spellcheck="false" style="display: inline; width: auto; height: 28px; padding-top: 0; padding-bottom: 2px; vertical-align:middle" class="form-control ui_password" type="password" ';
+    my ($name, $value, $size, $dis, $max, $tags, $class, $nostyle) = @_;
+    $class = $class ? " $class" : "";
+    my $style = ' style="display: inline; width: auto; height: 28px;'.
+                ' padding-top: 0; padding-bottom: 2px; vertical-align:middle"';
+    $style = '' if ($nostyle);
+    my $rv .=
+"<input$style class=\"form-control ui_password$class\" type=\"password\" ";
     $rv .= 'name="' . &quote_escape($name) . '" ';
     $rv .= 'value="' . &quote_escape($value) . '" ';
     $rv .= 'size="' . $size . '" ';
@@ -1180,10 +1203,11 @@ sub theme_ui_table_hr
 
 sub theme_ui_opt_textbox
 {
-    my ($name, $value, $size, $opt1, $opt2, $dis, $extra, $max, $tags) = @_;
+    my ($name, $value, $size, $opt1, $opt2, $dis, $extra, $max, $tags, $type) = @_;
     my $dis1 = &js_disable_inputs([$name, (defined($extra) ? @$extra : ())], []);
-    my $dis2 = &js_disable_inputs([],                                        [$name, (defined($extra) ? @$extra : ())]);
+    my $dis2 = &js_disable_inputs([], [$name, (defined($extra) ? @$extra : ())]);
     my $rv;
+    $type ||= 'text';
     $size = &ui_max_text_width($size);
     $rv .= &ui_radio($name . "_def",
                      $value eq '' ? 1 : 0,
@@ -1191,7 +1215,7 @@ sub theme_ui_opt_textbox
       "\n";
     my $min_width = $size ? '' : ' min-width: 15%;';
     $rv .=
-"<span><input class='ui_opt_textbox form-control' style='display: inline; width: auto; height: 28px; padding-top: 0; padding-bottom: 2px;$min_width' type='text' name=\""
+"<span><input class='ui_opt_textbox form-control' style='display: inline; width: auto; height: 28px; padding-top: 0; padding-bottom: 2px;$min_width' type='$type' name=\""
       . &quote_escape($name)
       . "\" " . "size=$size value=\"" .
       &quote_escape($value) . "\"" . ($dis ? " disabled=true" : "") . ($max ? " maxlength=$max" : "") .
@@ -1357,6 +1381,14 @@ sub theme_ui_radio_table
     return $rv;
 }
 
+sub theme_ui_make_date
+{
+    my ($date, $ts) = @_;
+    return $date if ($main::webmin_script_type ne 'web' ||
+                     $main::theme_prevent_make_date);
+    return "<span data-filesize-bytes='$ts'>$date</span>";
+}
+
 sub theme_make_date
 {
     return theme_make_date_local(@_);
@@ -1518,16 +1550,21 @@ sub theme_redirect_download
 
 sub theme_js_redirect
 {
-    my ($url, $window) = @_;
+    my ($url, $window, $timeout) = @_;
     $window ||= "window";
+    $timeout ||= 100;
     if ($url =~ /^\//) {
         # If the URL is like /foo , add webprefix
         $url = &get_webprefix().$url;
     }
-    return
-"<span class=\"loading-dots\"></span><script type='text/javascript'>var v___theme_postponed_fetcher = setTimeout(function(){ location.href = '"
-      . quote_escape($url)
-      . "';}, 2000);</script>\n";
+    return "<script type='text/javascript'>"."
+            try {
+                progress.end();
+                set_onbeforeunload_status_native(0);
+                set_onbeforeunload_status(0, 0);
+            } catch (e) {};
+            setTimeout(function(){location.href='@{[quote_escape($url)]}'},$timeout)".
+           "</script>\n";
 }
 
 sub theme_post_save_domain
@@ -1581,6 +1618,12 @@ sub theme_post_change_theme
 
     # Remove error handler
     error_40x_handler(1);
+    
+    # Remove service worker
+    print '<script>';
+    print 'navigator.serviceWorker.getRegistrations().then(
+        registrations => {registrations.forEach(reg => reg.unregister());});';
+    print "</script>\n";
 }
 
 sub theme_post_change_modules
@@ -1677,6 +1720,45 @@ sub theme_css_inline
     return get_css_inline(@_);
 }
 
+sub theme_forgot_url
+{
+    my ($baseurl, $id, $username) = @_;
+    my $url = "$baseurl/?forgot=".&urlize($id)."&username=".&urlize($username);
+    $url .= "&return=".&urlize($in{'return'}) if ($in{'return'});
+    return $url;
+}
+
+sub theme_forgot_handler
+{
+    my $page = shift;
+    if (!$ENV{'HTTP_X_REQUESTED_WITH'} && !$main::session_id) {
+        my $prefix = &get_webprefix()."/";
+        # If called from the framed theme
+        if ($page && $page =~ /forgot_form/ && $in{'return'}) {
+            my $return = &un_urlize($in{'return'});
+            &redirect($prefix."?return=".&urlize($return));
+            exit;
+        }
+        # Create expected format forgot link that is sent in email
+        elsif ($gconfig{'forgot_pass'} && $page =~ /forgot\.cgi$/) {
+            my ($id) = $ENV{'REQUEST_URI'} =~ /[?&]id=([0-9a-fA-F]{32})/;
+            if ($id) {
+                &redirect("$prefix?forgot=$id");
+                exit;
+            }
+        }
+        &redirect($prefix);
+        exit;
+    }
+}
+
+sub theme_ui_brh
+{
+    my $rv;
+    $rv .= "<div data-x-br></div>" . "\n";
+    return $rv;
+}
+
 $main::cloudmin_no_create_links = 1;
 $main::cloudmin_no_edit_buttons = 1;
 $main::cloudmin_no_global_links = 1;
