@@ -198,7 +198,8 @@ sub message_addressee
             $x_name = undef;
         }
         if ($x_address || $x_name) {
-            $x = ui_bold(html_escape($x_name || $x_address)) . html_escape(' &lt;' . trim($x_address || $x_name) . '&gt;');
+            $x = ui_text(html_escape($x_name)) . ($x_address ?
+                (' &lt;' . ui_text(html_escape(trim($x_address))) . '&gt;') : '');
         }
         push(@addressees_proc, $x);
     }
@@ -294,8 +295,8 @@ sub message_details
             $lock   = 'unlocked fa-flip-horizontal';
             $status = 'danger';
         }
-        my $success = ui_icon('check text-success');
-        my $fail    = ui_icon('times text-danger');
+        my $success = ui_mail_icon('check text-success');
+        my $fail    = ui_mail_icon('times text-danger');
 
         $data = ui_table_tbody_start('table table-condensed table-transparent table-message-details');
         my @tcontent;
@@ -318,7 +319,7 @@ sub message_details
         push(@tcontent, [$text{'extensions_mail_header_spf'} . ":", ui_italic(($spf ? $success : $fail), 'light')]);
         push(@tcontent,
              [$text{'extensions_mail_header_encrypted'} . ":",
-              ui_icon($encrypted)
+              ui_mail_icon($encrypted)
                 .
                 ui_italic(
                           (  $encryption ? $encryption_label :
@@ -346,9 +347,9 @@ sub message_flags
     my $unread  = 0;
     my $starred = $read & 2 ? 1 : 0;
     if ($starred) {
-        $special = ui_icon('star star', 'theme_xhred_global_starred');
+        $special = ui_mail_icon('star star', 'theme_xhred_global_starred');
     } else {
-        $special = ui_icon('star-o star', 'theme_xhred_global_unstarred');
+        $special = ui_mail_icon('star-o star', 'theme_xhred_global_unstarred');
     }
     if (($read & 1) == 0) {
         $unread = 1;
@@ -357,35 +358,37 @@ sub message_flags
     my @reply;
     my $reply;
     if ($read & 4) {
-        $reply = ui_icon('reply mail-list-reply');
+        $reply = ui_mail_icon('reply mail-list-reply');
     }
 
     my @security;
     my ($security_data, $security_status) = message_details($mail, $folder);
     my $security;
     if ($security_data) {
-        $security = ui_icon('caret-down text-light mail-list-dkim', $security_data, 'bottom');
+        $security = ui_mail_icon('caret-down text-light mail-list-dkim', $security_data, 'bottom');
     }
 
     my @all;
     my $all;
     my $p = int($mail->{'header'}->{'x-priority'});
     if ($p == 1 || $p == 2) {
-        $all .= ui_icon('exclamation text-danger mail-list-important', 'extensions_mail_flag_important');
+        $all .= ui_mail_icon('exclamation text-danger mail-list-important', 'extensions_mail_flag_important');
     } else {
-        $all .= ui_icon();
+        $all .= ui_mail_icon();
     }
 
     if (mail_has_attachments($mail, $folder)) {
-        $all .= ui_icon('paperclip fa-rotate-315 mail-list-attachment',
+        $all .= ui_mail_icon('paperclip fa-rotate-315 mail-list-attachment',
                         (
-                         ui_text($text{'extensions_mail_flag_attachment'} . "<br>(" . theme_nice_size_local($mail->{'size'}, 1024) . ")"
-                         )
+                         ui_text(
+                            html_escape($text{'extensions_mail_flag_attachment'} . 
+                                "<br>(" . theme_nice_size_local($mail->{'size'},
+                                    1024) . ")"))
                         ),
                         'auto top',
                         1);
     } else {
-        $all .= ui_icon();
+        $all .= ui_mail_icon();
     }
 
     my @dns;
@@ -394,15 +397,15 @@ sub message_flags
         open_dsn_hash();
         my $mid = $mail->{'header'}->{'message-id'};
         if ($dsnreplies{$mid}) {
-            $dns = ui_icon('read text-primary mail-list-dns-reply');
+            $dns = ui_mail_icon('read text-primary mail-list-dns-reply');
         }
         if ($delreplies{$mid}) {
             my ($bounce) = grep {/^\!/}
               split(/\s+/, $delreplies{$mid});
             if ($bounce) {
-                $dns = ui_icon('read text-danger mail-list-dns-reply');
+                $dns = ui_mail_icon('read text-danger mail-list-dns-reply');
             } else {
-                $dns = ui_icon('read text-success mail-list-dns-reply');
+                $dns = ui_mail_icon('read text-success mail-list-dns-reply');
             }
         }
     }
@@ -556,7 +559,7 @@ sub messages_list
         if ($showto) {
             $dcolumn .=
               ui_span_row('trow trow-to-pointer', 1) .
-              ui_span_row('trow trow-to') . ($showfrom ? ui_icon('long-arrow-right') : undef) .
+              ui_span_row('trow trow-to') . ($showfrom ? ui_mail_icon('long-arrow-right') : undef) .
               view_mail_link($folder, $id, $start, encode_guess($m->{'header'}->{'to'}, 'to')) . ui_span_row();
         }
         $dcolumn .= ui_span_row('trow trow-flag-security') . $flag_security . ui_span_row();
@@ -712,7 +715,7 @@ sub ui_tooltip
     return "data-tooltip='mailbox' data-placement='@{[$_[1] || 'bottom']}' data-title='$_[0]'";
 }
 
-sub ui_icon
+sub ui_mail_icon
 {
     my ($fa, $tt, $tp, $cb) = @_;
     if ($tt) {
@@ -739,14 +742,16 @@ sub ui_bold
 sub ui_italic
 {
     my ($str, $cls) = @_;
-    return '<em class=\'text-' . $cls . '\'>' . html_escape($str) . '</em>';
+    $str = filter_javascript($str);
+    return '<em class=\'text-' . $cls . '\'>' . $str . '</em>';
 
 }
 
 sub ui_text
 {
     my ($str, $cls) = @_;
-    return '<span class=\'text-' . $cls . '\'>' . html_escape($str) . '</span>';
+    $str = filter_javascript($str);
+    return '<span class=\'text-' . $cls . '\'>' . $str . '</span>';
 
 }
 
