@@ -542,8 +542,8 @@ sub nav_cat
                       'global_add'        => 'fa-plus',
                       'backup'            => 'fa-backup fa-1_15x',
                       'global_backup'     => 'fa-backup fa-1_15x',
-                      'global_server',    => 'fa-cogs',
-                      'cat_server',       => 'fa-cogs',
+                      'global_server'     => 'fa-cogs',
+                      'cat_server'        => 'fa-cogs',
                       'global_system'     => 'fa-cogs',
                       'global_delete'     => 'fa-plug',
                       'cat_delete'        => 'fa-plug',
@@ -589,9 +589,10 @@ sub nav_cat
 
 sub nav_search
 {
+    my $new_layout = shift;
     my $rv = "<li><br></li>";
     if (-r "$root_directory/webmin_search.cgi" && $gaccess{'webminsearch'}) {
-        $rv = "<li class=\"menu-container search-form-container\">\n";
+        $rv = "<li class=\"menu-container search-form-container new-layout-prod" . ($new_layout ? " new-layout" : "") . "\">\n";
         $rv .= "<form id=\"webmin_search_form\" action=\"$theme_webprefix/webmin_search.cgi\" role=\"search\">\n";
         $rv .= "<div class=\"form-group\">\n";
         $rv .=
@@ -698,6 +699,7 @@ sub nav_list_combined_menu
 {
     my ($modules, $items, $id, $group, $page) = @_;
     my ($nav_pos, $extra_links, $rv, $rv_after, $login_mode);
+    my $search_bar = '';
 
     my $gwp = sub {
         my ($link) = @_;
@@ -718,6 +720,26 @@ sub nav_list_combined_menu
         }
         return $link;
     };
+    
+    # Support upcoming change to navigation menu in Virtualmin
+    my $vm_new_format = grep { $_->{'format'} eq 'new' &&
+                               $_->{'module'} eq 'virtual-server' } @$items;
+    my $vm_has_menu = grep { $_->{'type'} eq 'menu' &&
+                             $_->{'module'} eq 'virtual-server' } @$items;
+    my $vm_has_new_domform = grep { $_->{'format'} eq 'link-new' &&
+                                    $_->{'module'} eq 'virtual-server' } @$items;
+    if ($vm_new_format) {
+        if ($vm_has_new_domform) {
+            my @vm_hr = grep { $_->{'type'} eq 'hr' &&
+                               $_->{'module'} eq 'virtual-server' } @$items;
+                    if (@vm_hr > 2) {
+                        $items = [ grep { $_ != $vm_hr[0] } @$items ];
+                        $items = [ grep { $_ != $vm_hr[1] } @$items ] if (!$vm_has_menu);
+                    }
+        }
+        $search_bar = nav_search($vm_new_format);
+    }
+
     foreach my $item (@$items) {
         if ((grep {$_ eq $item->{'module'}} @{$modules}) || $group) {
 
@@ -734,6 +756,9 @@ sub nav_list_combined_menu
                     $link =~ /server-manager\/index\.cgi/)
                 {
                     $icon = '<i class="fa fa-fw fa-tasks"></i>';
+                } elsif ($link =~ /\/virtualmin-wp-workbench/)
+                {
+                    $icon = '<i class="fa fa-fw fa2 fa2-workbench margined-left--05 margined-right-05"></i>';
                 } elsif ($link =~ /\/virtual-server\/edit_newvalidate\.cgi/ &&
                          !&webmin_user_is_admin())
                 {
@@ -774,13 +799,19 @@ sub nav_list_combined_menu
                 }
 
                 if ($link =~ /\/virtual-server\/domain_form\.cgi/) {
-                    $icon = '<i class="fa fa-fw fa-server-add"></i>';
+                    if ($link =~ /to=/) {
+                        $icon = '<i class="fa fa-fw fa-reply fa-flip-horizontal fa-0_90x margined-left--3 margined-right-3"></i>';
+                    } elsif ($link =~ /parentuser/) {
+                        $icon = '<i class="fa fa-fw fa-level-down fa-1_10x margined-left-1 margined-right--5"></i>';
+                    } else {
+                        $icon = '<i class="fa fa-fw fa-server-add"></i>';
+                    }
 
                 } elsif ($link =~ /\/virtual-server\/edit_domain\.cgi/ ||
                          $link =~ /\/server-manager\/edit_serv\.cgi/)
                 {
                     $icon = '<i class="fa fa-fw fa2 fa2-settings"></i>';
-                } elsif ($link =~ /\/virtual-server\/view_domain\.cgi/) {
+                } elsif ($link =~ /\/virtual-server\/(view_domain|summary_domain)\.cgi/ ) {
                     $icon = '<i class="fa fa-fw fa-info-circle"></i>';
 
                 } elsif ($link =~ /\/virtual-server\/list_users\.cgi/) {
@@ -818,33 +849,13 @@ sub nav_list_combined_menu
                     }
                 } elsif (
                         ($link =~ /^http:\/\// || $link =~ /^https:\/\// || $link =~ /^ftp:\/\// || $link =~ /^ftps:\/\//) &&
-                        $link !~ /virtualmin\.com\/professional/)
+                        $link !~ /virtualmin\.com\/docs\/professional/)
                 {
                     $icon = '<i class="fa fa-fw fa-external-link"></i>';
                 } elsif ($link =~ /\.\.\/servers\/link\.cgi/) {
                     $icon = '<i class="fa fa-fw fa-webmin scaled1_5"></i>';
                 }
 
-                # Print Virtual Server Summary link
-                my $print_server_summary = sub {
-                    my ($dom_id) = @_;
-                    return '<li data-linked><a target="page" class="navigation_module_trigger" href="' .
-                      $theme_webprefix . '/virtual-server/summary_domain.cgi?dom=' .
-                      $dom_id . '"><i class="fa fa-fw fa-info-circle"></i> <span>' .
-                      $theme_text{'right_vm_server_summary'} . '</span></a></li>' . "\n";
-                };
-                if ((&webmin_user_is_admin() || $get_user_level eq '1') &&
-                    $link =~ /\/virtual-server\/domain_form\.cgi/ &&
-                    nav_virtualmin_domain_available_count())
-                {
-                    my $dom_id = $item->{'link'};
-                    $dom_id =~ /gparent=(\d+)/;
-                    $dom_id = $1;
-                    if ($dom_id) {
-                        $rv .= &$print_server_summary($dom_id);
-                    }
-                }
-
                 # Set variable in case it hasn't been set before
                 if (!length $link) {
                     $icon = undef;
@@ -927,8 +938,17 @@ sub nav_list_combined_menu
                 }
                 $rv .= "</ul></li>\n";
             } elsif ($item->{'type'} eq 'hr') {
-                if ($nav_pos++ eq '1') {
-                    $rv .= nav_search();
+                $nav_pos++;
+                my $separator = '<li class="menu-container menu-separator"><span class="hr"></span></li>';
+                if ($vm_new_format) {
+                    if ($nav_pos eq '2') {
+                        $rv .= $separator;
+                    }
+                }
+                else {
+                    if ($nav_pos eq '2') {
+                        $rv .= nav_search($vm_new_format);
+                    }
                 }
             } elsif (($item->{'type'} eq 'menu' || $item->{'type'} eq 'input') &&
                      $item->{'module'} ne 'mailbox')
@@ -944,7 +964,7 @@ sub nav_list_combined_menu
                     $rv .= ui_hidden(@$h);
                 }
 
-                $rv .= $item->{'desc'}, "\n";
+                $rv .= $item->{'desc'};
                 if ($item->{'type'} eq 'menu' || $item->{'type'} eq 'input') {
                     if ($item->{'name'} eq 'dname' && $item->{'module'} eq 'virtual-server') {
 
@@ -974,7 +994,7 @@ sub nav_list_combined_menu
             }
         }
     }
-    return { 'before' => $rv,
+    return { 'before' => $search_bar . $rv,
              'after'  => $rv_after,
              'mode'   => $login_mode };
 }
