@@ -20,6 +20,58 @@ sub check_firewalld
 return undef;
 }
 
+# get_config_files()
+# Returns a list of all firewalld config files
+sub get_config_files
+{
+my $conf_dir = $config{'config_dir'} || '/etc/firewalld';
+return (glob("$conf_dir/*.xml"), glob("$conf_dir/*/*.xml"));
+}
+
+# check_ip_family()
+# Determines which IP families are enabled and functional on the system
+#
+# Returns:
+#   Hash with 'ipv4' and 'ipv6' keys set to 1 if enabled, 0 if disabled, -1 if unknown
+sub check_ip_family
+{
+my %result = ( ipv4 => 0, ipv6 => 0 );
+
+# Attempt to load IO::Socket::IP at runtime
+eval {
+	require IO::Socket::IP;
+		IO::Socket::IP->import();
+		1;
+	};
+
+# Return on error (must never happen on contemporary systems)
+if ($@) {
+	$result{ipv4} = -1; # unknown
+	$result{ipv6} = -1; # unknown
+	return %result;
+	}
+
+# Check IPv4
+eval {
+	my $sock4 = IO::Socket::IP->new(
+		LocalHost => '127.0.0.1',
+		LocalPort => 0,  # ephemeral port
+		Proto     => 'tcp');
+	$result{ipv4} = 1 if $sock4;
+	};
+
+# Check IPv6
+eval {
+	my $sock6 = IO::Socket::IP->new(
+		LocalHost => '::1',
+		LocalPort => 0,  # ephemeral port
+		Proto     => 'tcp' );
+	$result{ipv6} = 1 if $sock6;
+	};
+
+return %result;
+}
+
 # is_firewalld_running()
 # Returns 1 if the server is running, 0 if not
 sub is_firewalld_running
@@ -91,8 +143,8 @@ $service =~ s/[^A-Za-z0-9\-]//g;
 # Check for file in directory containing all services as xml files
 my @ports;
 my @protos;
-foreach my $services_dir ("/usr/lib/firewalld/services",
-			  "/etc/firewalld/services") {
+foreach my $services_dir ("/etc/firewalld/services",
+			  "/usr/lib/firewalld/services") {
 	my $service_file = "$services_dir/$service.xml";
 	if (-r $service_file) {
 		my $lref = &read_file_lines($service_file, 1);
@@ -104,6 +156,7 @@ foreach my $services_dir ("/usr/lib/firewalld/services",
 				push(@protos, $proto) if ($port && $proto);
 				}
 			}
+		last if (@ports);
 		}
 	}
 @ports = &unique(@ports);
@@ -384,141 +437,335 @@ my ($zone) = grep { $_->{'default'} } @zones;
 return $zone;
 }
 
-# add_ip_ban(ip, [zone])
-# Ban given IP address in given or default zone
-sub add_ip_ban
+# construct_rich_rule([@opts])
+# Constructs rich firewalld rule and returns it as string
+#
+# Parameters:
+#   @opts - Array of key-value pairs defining the rule properties
+# 
+# Example:
+#   &construct_rich_rule(
+#       'source address' => '0.0.0.0/0',
+#       'log prefix' => 'BANDWIDTH_IN ',
+#       'level' => 'info',
+#       'action' => 'accept',
+#   );
+# 
+# Returns:
+#   A string representing the rich rule
+sub construct_rich_rule
 {
-my ($ip, $zone) = @_;
-return create_rich_rule('add', $ip, $zone);
-}
+my (@opts) = @_;
+my %opts = @opts;
 
-# remove_ip_ban(ip, [zone])
-# Un-ban given IP address in given or default zone 
-sub remove_ip_ban
-{
-my ($ip, $zone) = @_;
-return create_rich_rule('remove', $ip, $zone);
-}
-
-# create_rich_rule(action, ip, [\zone], [opts])
-# Add or remove rich rule for given IP in given or default zone 
-sub create_rich_rule
-{
-my ($action, $ip, $zone, $opts) = @_;
-my $ip_validate = sub {
-	return &check_ipaddress($_[0]) || &check_ip6address($_[0]);
-	};
-
-# Default action for permanent ban is 'drop'
-my $action_type = "drop";
-
-# Override defaults
-if (ref($opts)) {
-	
-	# Override default action
-	$action_type = lc($opts->{'action'})
-		if ($opts->{'action'} &&
-		    $opts->{'action'} =~ /^accept|reject|drop|mark$/);
-}
-
-# Zone name
-if (!$zone) {
-	($zone) = get_default_zone();
+# Action type
+my $action_type = $config{'packet_handling'} eq '1' ? 'reject' : 'drop';
+my $opts_action = delete($opts{'action'});
+if ($opts_action) {
+	$action_type = lc($opts_action)
+		if ($opts_action &&
+		    $opts_action =~ /^accept|reject|drop|mark$/);
 	}
-$zone = $zone->{'name'};
+
+# Set family
+my $family = delete($opts{'family'}) || 'ipv4';
+
+# Validate IP addresses, and update family if needed
+foreach my $ip_key ('source address', 'destination address') {
+	if (my $full_ip = $opts{$ip_key}) {
+		# Split IP and CIDR, if present
+		my ($ip_only, $cidr) = split(/\//, $full_ip);
+
+		# Validate the IP portion
+		&check_ipaddress($ip_only) || &check_ip6address($ip_only) ||
+			&error("$text{'list_rule_iperr'} : $ip_only");
+
+		# Decide family based on presence of ':' in IP portion
+		$family = $ip_only =~ /:/ ? 'ipv6' : 'ipv4';
+
+		# If you still want to test or store the CIDR, do it here
+		if (defined($cidr)) {
+			# Make sure CIDR is numeric and within range
+			$cidr =~ /^\d+$/ && $cidr <= ($family eq 'ipv6' ? 128 : 32) ||
+				&error("$text{'save_rule_cidrerr'} : /$cidr");
+			}
+		}
+	}
+
+# Construct rule from given options
+my $rule = "rule family=\"".quotemeta($family)."\"";
+
+# Extended options handling for dynamic rules
+foreach my $key (@opts) { # Iterate over all keys in given order
+	next if (!defined($opts{$key}));
+	# Keys cannot be quotemeta'd
+	$key =~ tr/A-Za-z0-9\-\_\/ //cd;
+	# Values can be quotemeta'd
+	my $val = $opts{$key};
+	$val =~ tr/A-Za-z0-9\-\_\=\'\:\.\,\/ //cd;
+	$rule .= " $key=\"$val\"";
+	}
+$rule .= " ".quotemeta($action_type);
+return $rule;
+}
+
+# rich_rule(action, [@opts])
+# Adds or removes a rich firewalld rule with specified options and rich rule
+# passed as string
+#
+# Parameters:
+#   action - Required. Must be either 'add' or 'remove'
+#   @opts  - Array of key-value pairs defining the rule properties
+# 
+# Example:
+#   &rich_rule('add', {
+#       'zone' => 'public',
+#       'rule' => &construct_rich_rule('source address' => '0.0.0.0/0'),
+#       'permanent' => 1,
+#   });
+# 
+# Returns:
+#   undef on success, or (error_message, error_code) on failure in list context
+sub rich_rule
+{
+my ($action, $opts) = @_;
 
 # Validate action
 $action eq 'add' || $action eq 'remove' || &error($text{'list_rule_actionerr'});
 
-# Validate IP
-&$ip_validate($ip) || &error($text{'list_rule_iperr'});
+# Zone name
+my $zone = $opts->{'zone'};
+if (!$zone) {
+	($zone) = get_default_zone();
+	$zone = $zone->{'name'};
+	}
 
-# Set family
-my $family = $ip =~ /:/ ? 'ipv6' : 'ipv4';
+# Permanent rule
+my $permanent = $opts->{'permanent'};
 
 # Add/remove rich rule
 my $get_cmd = sub {
 	my ($rtype) = @_;
-	my $type;
-	$type = " --permanent" if ($rtype eq 'permanent');
-	return "$config{'firewall_cmd'} --zone=".quotemeta($zone)."$type --".quotemeta($action)."-rich-rule=\"rule family=".quotemeta($family)." source address=".quotemeta($ip)." ".quotemeta($action_type)."\"";
+	my $type = $rtype ? " --permanent" : "";
+	return "$config{'firewall_cmd'} --zone=\"".quotemeta($zone)."\"".
+	       "$type --".quotemeta($action)."-rich-rule='$opts->{'rule'}'";
 	};
-my $out = &backquote_logged(&$get_cmd()." 2>&1 </dev/null");
-return $out if ($?);
-$out = &backquote_logged(&$get_cmd('permanent')." 2>&1 </dev/null");
-return $? ? $out : undef;
+
+for my $type (0..1) {
+	next if ($type == 1 && !$permanent);
+	my $cmd = &$get_cmd($type);
+	my $out = &backquote_logged($cmd." 2>&1 </dev/null");
+	return wantarray ? ($out, $?) : $out if ($?);
+	}
+return undef;
 }
 
-# remove_rich_rule(rule, [\zone])
-# Remove rich rule in given or default zone 
-sub remove_rich_rule
+# check_rich_rule(rule, [&zone])
+# Check if a rich rule exists in the given or default zone
+sub check_rich_rule
 {
 my ($rule, $zone) = @_;
 
 # Zone name
 if (!$zone) {
 	($zone) = get_default_zone();
+	$zone = $zone->{'name'};
 	}
-$zone = $zone->{'name'};
 
-# Remove rule command
-my $get_cmd = sub {
-	my ($rtype) = @_;
-	my $type;
-	$type = " --permanent" if ($rtype eq 'permanent');
-	return "$config{'firewall_cmd'} --zone=".quotemeta($zone)."$type --remove-rich-rule ".quotemeta(&trim($rule))."";
-	};
+# Command to query rules
+my $cmd = "$config{'firewall_cmd'} --zone=".quotemeta($zone)." --list-rich-rules";
+my $out = &backquote_logged($cmd." 2>&1 </dev/null");
 
-my $out = &backquote_logged(&$get_cmd()." 2>&1 </dev/null");
-return $out if ($?);
-$out = &backquote_logged(&$get_cmd('permanent')." 2>&1 </dev/null");
-return $? ? $out : undef;
+# Check for rule existence
+return ($out =~ /\Q$rule\E/);
 }
 
-# remove_direct_rule(rule)
-# Remove given direct rule
-sub remove_direct_rule
+# add_rich_rule(rule, [&zone])
+# Add rich rule in given or default zone using raw rule string
+sub add_rich_rule
+{
+my ($rule, $zone) = @_;
+return &rich_rule('add',
+	{ 'zone' => $zone->{'name'}, 'permanent' => 1, 'rule' => $rule });
+}
+
+# remove_rich_rule(rule, [&zone])
+# Remove rich rule in given or default zone using raw rule string
+sub remove_rich_rule
+{
+my ($rule, $zone) = @_;
+return &rich_rule('remove',
+	{ 'zone' => $zone->{'name'}, 'permanent' => 1, 'rule' => $rule });
+}
+
+# construct_direct_rule(&opts)
+# Constructs a direct Firewalld rule string
+#
+# Opts can include:
+#   'family'   => 'ipv4' | 'ipv6' | 'eb'  (default = 'ipv4')
+#   'table'    => 'filter' | 'nat' | 'mangle' | 'raw' |
+#                 'security' (default = 'filter')
+#   'chain'    => 'INPUT' | 'OUTPUT' | 'FORWARD' |
+#                 'PREROUTING' | 'POSTROUTING' (default = 'INPUT')
+#   'priority' => integer priority (default = 0)
+#   'rule'     => string containing iptables-like match/target
+#
+# Returns:
+#   A string representing the direct rule is returned
+#
+sub construct_direct_rule
+{
+my ($opts) = @_;
+
+# Defaults
+my $family   = $opts->{'family'}   || 'ipv4';
+my $table    = $opts->{'table'}    || 'filter';
+my $chain    = $opts->{'chain'}    || 'INPUT';
+my $priority = $opts->{'priority'} // 0;
+my $rule     = $opts->{'rule'}     || '';
+
+# Basic validation
+$family =~ /^(ipv4|ipv6|eb)$/ ||
+	&error(&text('save_rule_efamily', $family));
+
+$table =~ /^(filter|nat|mangle|raw|security)$/ ||
+	&error(&text('save_rule_etable', $table));
+
+$chain =~ /^(INPUT|OUTPUT|FORWARD|PREROUTING|POSTROUTING)$/ ||
+	&error(&text('save_rule_echain', $chain));
+
+# Priority must be integer
+$priority =~ /^\d+$/ || &error(&text('save_rule_epriority', $priority));
+
+# If still empty after parsing, throw an error
+$rule !~ /^\s*$/ || &error(&text('save_rule_erule'));
+
+# Sanitize rule string by splitting into components and validating each
+my @parts = split(/\s+/, $rule);
+my $sanitized_rule = '';
+for (my $i = 0; $i < @parts; $i++) {
+	my $part = $parts[$i];
+	next if (!defined($part) || $part eq '');
+
+	if ($family =~ /^ipv[46]$/ &&
+	    $part =~ /^(?:--source|--destination)$/) {
+		# Get the IP value (next part)
+		my $ip = $parts[++$i];
+		if (defined($ip)) {
+			# Split IP and CIDR if present
+			my ($ip_only, $cidr) = split(/\//, $ip);
+
+			# Validate the IP portion
+			&check_ipaddress($ip_only) ||
+			&check_ip6address($ip_only) ||
+				&error("$text{'list_rule_iperr'} : $ip_only");
+
+			# Verify IP family matches the rule family
+			my $ip_family = $ip_only =~ /:/ ? 'ipv6' : 'ipv4';
+			$ip_family eq $family ||
+				&error(&text('save_rule_eruleipmismatch'));
+
+			# Validate CIDR if present
+			if (defined($cidr)) {
+				# Make sure CIDR is numeric and within range
+				my $cidr_valid = $family eq 'ipv6' ? 128 : 32;
+				$cidr =~ /^\d+$/ && $cidr <= ($cidr_valid) ||
+					&error("$text{'save_rule_cidrerr'} : /$cidr");
+				}
+			$sanitized_rule .= ' ' . $part . ' ' . $ip;
+			}
+		}
+	elsif ($family eq 'eb' && $part =~ /^(?:--src-mac|--dst-mac)$/) {
+		# Get the MAC value (next part)
+		my $mac = $parts[++$i];
+		if (defined($mac)) {
+			# MAC validation could be added here
+			$sanitized_rule .= ' ' . $part . ' ' . $mac;
+			}
+		}
+	else {
+		if ($part =~ /^-/) {
+			# Options/flags can only contain certain characters
+			$part =~ tr/A-Za-z0-9\-\_//cd;
+			}
+		else {
+			# Values can contain more characters
+			$part =~ tr/A-Za-z0-9\-\_\=\.\:\,\/\"\'//cd;
+			}
+		$sanitized_rule .= ' ' . $part;
+		}
+	}
+
+# Remove extra possible spaces
+$sanitized_rule =~ s/\s+/ /g;
+
+# Return the constructed rule
+return "$family $table $chain $priority $sanitized_rule";
+}
+
+# direct_rule(action, &opts)
+# Add or remove a direct rule
+#
+# Returns:
+#   undef on success, or (error_message, error_code) on failure in list context
+sub direct_rule
+{
+my ($action, $opts) = @_;
+
+# Validate action
+$action eq 'add' || $action eq 'remove' || &error($text{'list_rule_actionerr'});
+
+# Extract permanent flag and construct rule
+my $permanent = delete($opts->{'permanent'});
+
+# Get rule
+my $rule = $opts->{'rule'};
+$rule =~ s/\s+/ /g;
+
+# Add/remove direct rule
+my $get_cmd = sub {
+	my ($perm) = @_;
+	my $type = $perm ? " --permanent" : "";
+	return "$config{'firewall_cmd'}$type --direct --$action-rule $rule";
+	};
+
+for my $type (0..1) {
+	next if ($type == 1 && !$permanent);
+	my $cmd = &$get_cmd($type);
+	my $out = &backquote_logged($cmd." 2>&1 </dev/null");
+	return wantarray ? ($out, $?) : $out if ($?);
+	}
+return undef;
+}
+
+# check_direct_rule(rule)
+# Check if a direct rule exists
+#
+# Returns:
+#   1 if rule exists, 0 if not
+sub check_direct_rule
 {
 my ($rule) = @_;
 
-# Sanitize rule manually (couldn't make it work with quotemeta)
-$rule =~ tr/A-Za-z0-9\-\_\=\"\:\.\,\/ //cd;
+# Construct rule for matching
+my ($family, $table, $chain) = split(/\s+/, $rule);
+my $rrule = $rule;
+$rrule =~ s/^\Q$family\E\s+\Q$table\E\s+\Q$chain\E\s+//;
+$rrule =~ s/\s+/ /g;
 
-# Remove rule command
-my $get_cmd = sub {
-	my ($rtype) = @_;
-	my $type;
-	$type = " --permanent" if ($rtype eq 'permanent');
-	return "$config{'firewall_cmd'}${type} --direct --remove-rule ".&trim($rule)."";
-	};
-
-my $out = &backquote_logged(&$get_cmd()." 2>&1 </dev/null");
-return $out if ($?);
-$out = &backquote_logged(&$get_cmd('permanent')." 2>&1 </dev/null");
-return $? ? $out : undef;
+# Get existing rules
+my $cmd = "$config{'firewall_cmd'} --direct --get-rules $family $table $chain";
+my $out = &backquote_logged($cmd." 2>&1 </dev/null");
+return $? ? 0 : ($out =~ /\Q$rrule\E/);
 }
 
-sub get_config_files
+# remove_direct_rule(rule)
+# Remove given direct rule passed as string
+sub remove_direct_rule
 {
-my $conf_dir = $config{'config_dir'} || '/etc/firewalld';
-return (glob("$conf_dir/*.xml"), glob("$conf_dir/*/*.xml"));
-}
-
-# block_ip(ip, zone, [permanent])
-# Block given IP address temporarily or permanently
-sub block_ip
-{
-my ($ip, $zone, $permanent) = @_;
-my $type = $permanent ? ' --permanent' : "";
-my $family = $ip =~ /:/ ? 'ipv6' : 'ipv4';
-my $handle_action = $config{'packet_handling'} eq '1' ? 'reject' : 'drop';
-my $out = &backquote_logged(
-	"$config{'firewall_cmd'}$type --zone=".quotemeta($zone).
-		 " --add-rich-rule=\"rule family=$family source address=".
-		 	quotemeta($ip)." $handle_action\" 2>&1");
-my $rs = $? ? $out : undef;
-&apply_firewalld() if ($permanent);
-return $rs;
+my ($rule) = @_;
+my ($out, $rs) = &direct_rule('remove', { 'rule' => $rule, 'permanent' => 1 });
+return wantarray ? ($out, $?) : $out if ($?);
 }
 
 1;
