@@ -650,13 +650,14 @@ return $rv;
 
 sub quote_table
 {
-return &quotestr($_[0]);
+my ($rv) = @_;
+return &quotestr($rv);
 }
 
 # quotestr(string)
 sub quotestr
 {
-local $rv = $_[0];
+my ($rv) = @_;
 if (&supports_quoting()) {
 	return "`$rv`";
 	}
@@ -677,18 +678,19 @@ return $db;
 
 # unquote_mysql_database(name)
 # Returns a MySQL database name with \% and \_ characters unescaped
-sub unquote_mysql_database {
-    my ($db) = @_;
-    $db =~ s/\\%/%/g;
-    $db =~ s/\\_/_/g;
-    return $db;
+sub unquote_mysql_database
+{
+my ($db) = @_;
+$db =~ s/\\%/%/g;
+$db =~ s/\\_/_/g;
+return $db;
 }
 
 # escapestr(string)
 # Returns a string with quotes escaped, for use in SQL
 sub escapestr
 {
-local $rv = $_[0];
+my ($rv) = @_;
 $rv =~ s/'/''/g;
 return $rv;
 }
@@ -697,8 +699,8 @@ return $rv;
 # Escapes backslashes, but only inside quoted strings
 sub escape_backslashes_in_quotes
 {
-local ($str) = @_;
-local $rv;
+my ($str) = @_;
+my $rv;
 while($str =~ /^([^"]*)"([^"]*)"(.*)$/) {
 	local ($before, $quoted, $after) = ($1, $2, $3);
 	$quoted =~ s/\\/\\\\/g;
@@ -724,6 +726,14 @@ sub supports_mysqldump_events
 return &compare_version_numbers($mysql_version, "5.1.8") >= 0;
 }
 
+# supports_mysqldump_setgtid()
+# Returns 1 if mysqldump supports --set-gtid-purged flag
+sub supports_mysqldump_setgtid
+{
+my $out = &backquote_command("$config{'mysqldump'} --help 2>&1 </dev/null");
+return $out =~ /--set-gtid-purged/ ? 1 : 0;
+}
+
 # supports_routines()
 # Returns 1 if mysqldump supports routines
 sub supports_routines
@@ -1149,7 +1159,7 @@ sub get_mysql_config
 {
 if (!scalar(@mysql_config_cache)) {
 	if (!-r $config{'my_cnf'}) {
-		return undef;
+		return [];
 		}
 	@mysql_config_cache = &parse_mysql_config($config{'my_cnf'});
 	}
@@ -1168,9 +1178,12 @@ local $lref = &read_file_lines($file, 1);
 local $_;
 foreach (@$lref) {
 	s/\r|\n//g;
-	s/#.*$//;
 	s/\s+$//;
-	if (/^\s*\[(\S+)\]$/) {
+	if (/^\s*(#|;)/) {
+		$lnum++;
+		next;
+		}
+	elsif (/^\s*\[(\S+)\]$/) {
 		# Start of a section
 		$sect = { 'name' => $1,
 			  'members' => [ ],
@@ -1346,6 +1359,26 @@ my $rs = &execute_sql_safe($db, $sql_query, $db, @tables);
 return $rs;
 }
 
+# get_all_tables_size(db)
+# Retrieves the size of all tables in the given database
+sub get_all_tables_size
+{
+my ($db) = @_;
+my @tables = list_tables($db);
+my $sql_query = "
+    SELECT
+        TABLE_SCHEMA,
+        TABLE_NAME,
+        ENGINE,
+        DATA_LENGTH + INDEX_LENGTH AS total_size_bytes
+    FROM information_schema.TABLES
+    WHERE TABLE_SCHEMA = ?
+      AND TABLE_NAME IN (" . join(", ", ("?") x @tables) . ")
+";
+my $rs = &execute_sql_safe($db, $sql_query, $db, @tables);
+return $rs;
+}
+
 # list_indexes(db)
 # Returns the names of all indexes in some database
 sub list_indexes
@@ -1646,7 +1679,8 @@ my $dropsql = $drop ? "--add-drop-table" : "";
 my $singlesql = $single ? "--single-transaction" : "";
 my $forcesql = $force ? "--force" : "";
 my $quicksql = $quick ? "--quick" : "";
-my $parameterssql = $parameters ? quotemeta($parameters) : "";
+my $parameterssql = $parameters ?
+	join(" ", map { quotemeta($_) } split(/\s+/, $parameters)) : "";
 my $wheresql = $where ? "--where=".quotemeta($in{'where'}) : "";
 my $charsetsql = $charset ?
 	"--default-character-set=".quotemeta($charset) : "";
@@ -1657,15 +1691,21 @@ my $routinessql = &supports_routines() ? "--routines" : "";
 my $tablessql = join(" ", map { quotemeta($_) } @$tables);
 my $eventssql = &supports_mysqldump_events() ? "--events" : "";
 my $gtidsql = "";
-eval {
-	local $main::error_must_die = 1;
-	my $d = &execute_sql($master_db, "show variables like 'gtid_mode'");
-	if (@{$d->{'data'}} && uc($d->{'data'}->[0]->[1]) eq 'ON' &&
-	    &compare_version_numbers($mysql_version, "5.6") >= 0) {
-		# Add flag to support GTIDs
-		$gtidsql = "--set-gtid-purged=OFF";
-		}
-	};
+if (&supports_mysqldump_setgtid() &&
+    $config{'mysqldump'} !~ /--set-gtid-purged/) {
+	eval {
+		local $main::error_must_die = 1;
+		my $d = &execute_sql($master_db,
+			"show variables like 'gtid_mode'");
+		my ($ver, $variant) = &get_remote_mysql_variant();
+		if (@{$d->{'data'}} && uc($d->{'data'}->[0]->[1]) eq 'ON' &&
+		    $variant eq 'mysql' &&
+		    &compare_version_numbers($ver, "5.6") >= 0) {
+			# Add flag to support GTIDs
+			$gtidsql = "--set-gtid-purged=OFF";
+			}
+		};
+	}
 if ($user && $user ne "root") {
 	# Actual writing of output is done as another user
 	$writer = &command_as_user($user, 0, $writer);
@@ -1706,45 +1746,89 @@ my $conf = &get_mysql_config();
 return &unique(map { $_->{'file'} } @$conf);
 }
 
-# get_change_pass_sql(unescaped_plaintext_password, user, host)
+# get_account_lock_status(user, host)
+# Returns the account lock status of a user
+sub get_account_lock_status
+{
+my ($user, $host) = @_;
+my $rv = &execute_sql_safe($master_db, 'show create user ?@?', $user, $host);
+return undef if (!ref($rv) || !@{$rv->{'data'}});
+return $rv->{'data'}->[0][0] =~ /account\s+lock/i ? 1 : 0;
+}
+
+# get_account_lock_support()
+# Returns 1 if the MySQL/MariaDB server supports account locking
+sub get_account_lock_support
+{
+my ($ver, $variant) = &get_remote_mysql_variant();
+return 
+   $variant eq "mariadb" && &compare_version_numbers($ver, "10.4.2") >= 0 ||
+   $variant eq "mysql" && &compare_version_numbers($ver, "8.0") >= 0;
+}
+
+# get_plugin_sql(version, variant, plainpass, plugin)
+# Get the right query for setting user password with plugin
+sub get_plugin_sql
+{
+my ($ver, $variant, $plainpass, $plugin) = @_;
+my $pass = &escapestr($plainpass);
+# Has account locking support?
+my $suplock = &get_account_lock_support();
+my $lockcurr;
+if ($suplock) {
+	$lockcurr = !defined($plainpass);
+	if ($lockcurr) {
+		$pass = sprintf("%x", rand 16) for 1..30;
+		}
+	}
+my $is_plugin_socket = $plugin eq "unix_socket";
+my $by = "";
+$by = " by '$pass'" if (!$is_plugin_socket);
+my $sp = "identified with $plugin$by";
+if ($variant eq "mariadb") {
+	$by = " using $password_func('$pass')" if (!$is_plugin_socket);
+	$sp = "identified via $plugin$by";
+	}
+if ($suplock) {
+	$sp = $lockcurr ? "account lock" : "$sp account unlock";
+	}
+return $sp;
+}
+
+# get_change_pass_sql(unescaped_plaintext_password, user, host, plugin)
 # Get the right query for changing user password
 sub get_change_pass_sql
 {
-my ($unescaped_plainpass, $user, $host) = @_;
-my $plugin = &get_mysql_plugin(1);
-my $escaped_pass = &escapestr($unescaped_plainpass);
+my ($unescaped_plainpass, $user, $host, $plugin) = @_;
+$plugin ||= &get_mysql_plugin();
 my $sql;
 my ($ver, $variant) = &get_remote_mysql_variant();
-my $mysql_mariadb_with_auth_string = 
+my $supauth = 
    $variant eq "mariadb" && &compare_version_numbers($ver, "10.2") >= 0 ||
    $variant eq "mysql" && &compare_version_numbers($ver, "5.7.6") >= 0;
-if ($mysql_mariadb_with_auth_string && $unescaped_plainpass) {
-	$sql = "alter user '$user'\@'$host' identified $plugin by '$escaped_pass'";
+if ($plugin && $supauth) {
+	my $sp = &get_plugin_sql($ver, $variant, $unescaped_plainpass, $plugin);
+	$sql = "alter user '$user'\@'$host' $sp";
 	}
 else {
+	my $escaped_pass = &escapestr($unescaped_plainpass);
 	$sql = "set password for '".$user."'\@'".$host."' = ".
 	       "$password_func('$escaped_pass')";
 	}
 return $sql;
 }
 
-# get_mysql_plugin(query_ready)
+# get_mysql_plugin()
 # Returns the name of the default plugin used by MySQL/MariaDB
 sub get_mysql_plugin
 {
-my ($query) = @_;
 if ($config{'auth_plugin'}) {
-	return " with $config{'auth_plugin'}";
-	}
-else {
-	my @plugin = &execute_sql($master_db, 
-	    "show variables LIKE '%default_authentication_plugin%'");
-	my $plugin = $plugin[0]->{'data'}->[0]->[1];
-	if ($plugin && $query) {
-		$plugin = " with $plugin ";
-		}
-	return $plugin;
+	return $config{'auth_plugin'};
 	}
+my $rv = &execute_sql($master_db, 
+    "show variables LIKE '%default_authentication_plugin%'");
+return undef if (!ref($rv) || !@{$rv->{'data'}});
+return $rv->{'data'}->[0]->[1];
 }
 
 # perms_column_to_privilege_map(col)
@@ -1874,7 +1958,8 @@ my $other_field_names = $sc->{'other_field_names'};
 my $other_field_values = $sc->{'other_field_values'};
 
 my ($ver, $variant) = &get_remote_mysql_variant();
-my $plugin = &get_mysql_plugin(1);
+my $plugin = $sc->{'plugin'} || &get_mysql_plugin();
+$plugin = $plugin ? "with $plugin" : "";
 
 if ($variant eq "mariadb" && &compare_version_numbers($ver, "10.4") >= 0) {
 	my $sql = "create user '$user'\@'$host' identified $plugin by ".
@@ -1912,38 +1997,17 @@ else {
 	}
 }
 
-# change_user_password(plainpass, user, host)
+# change_user_password(plainpass, user, host, plugin)
 # Change user password
 sub change_user_password
 {
-my ($plainpass, $user, $host) = @_;
-
-my ($ver, $variant) = &get_remote_mysql_variant();
-my $plugin = &get_mysql_plugin(1);
-my $lock_supported = $variant eq "mysql" && &compare_version_numbers($ver, "8.0.19");
-my $mysql_mariadb_with_auth_string = 
-	$variant eq "mariadb" && &compare_version_numbers($ver, "10.4") >= 0 ||
-	$variant eq "mysql" && &compare_version_numbers($ver, "5.7.6") >= 0;
-
+my ($plainpass, $user, $host, $plugin) = @_;
+$plugin ||= &get_mysql_plugin();
+$plugin ||= "";
 my $sql;
-my $pass = &escapestr($plainpass);
 $host ||= '%';
-my $lock = !defined($plainpass);
-if ($lock) {
-	$pass = sprintf("%x", rand 16) for 1..30;
-	}
-if ($mysql_mariadb_with_auth_string) {
-	my $sp = "identified $plugin by '".$pass."'";
-	if ($lock_supported) {
-		$sp = $lock ? "account lock" : "$sp account unlock";
-		}
-	$sql = "alter user '$user'\@'$host' $sp";
-	&execute_sql_logged($master_db, $sql);
-	}
-else {
-	$sql = &get_change_pass_sql($plainpass, $user, $host);
-	&execute_sql_logged($master_db, $sql);
-	}
+$sql = &get_change_pass_sql($plainpass, $user, $host, $plugin);
+&execute_sql_logged($master_db, $sql);
 
 # Update module password when needed
 &update_config_credentials({
@@ -2037,7 +2101,7 @@ if (!$pid || !kill(0, $pid)) {
 	}
 
 # Update password by running command directly
-my $cmd = $config{'mysql'} || 'mysql';
+$cmd = $config{'mysql'} || 'mysql';
 my $sql = &get_change_pass_sql($pass, $user, 'localhost');
 my $out = &backquote_command("$cmd -D $master_db -e ".
 		quotemeta("flush privileges; $sql")." 2>&1 </dev/null");
@@ -2132,5 +2196,100 @@ return 'password' if ($@);	# Old version without plugins
 return $rv->{'data'}->[0]->[0] =~ /unix_socket/i ? 'socket' : 'password';
 }
 
+# list_authentication_plugins()
+# Returns a list ref of supported authentication plugins for setting passwords
+sub list_authentication_plugins
+{
+my ($ver, $variant) = &get_remote_mysql_variant();
+if ($variant eq "mariadb" && &compare_version_numbers($ver, "10.4") >= 0 ||
+    $variant eq "mysql" && &compare_version_numbers($ver, "5.7.6") >= 0) {
+	my $rv = &execute_sql($master_db, "show plugins");
+	my @plugins = map { $_->[0] } grep { $_->[1] eq 'ACTIVE' &&
+		$_->[2] eq 'AUTHENTICATION' } @{ $rv->{data} };
+	return @plugins ? \@plugins : ['mysql_native_password'];
+	}
+return ();
+}
+
+# format_privs(&privs, &privs_fields)
+# Returns best formatted string for a set of privileges
+sub format_privs
+{
+my ($privs, $privs_fields) = @_;
+my @privs_all = map { lc($_->[1]) } @{$privs_fields};
+my @privs_cur = map { lc($_) } @{$privs};
+my $simplify_privs = sub {
+	my @privs = @_;
+	my %groups = (
+		'table_data' => [],
+		'tables' => [],
+		'view' => [],
+		'routine' => [],
+		'replication' => [],
+	);
+	my @others = ();
+	foreach my $priv (@privs) {
+		if ($priv =~ /^(select|insert|update|delete) table data$/) {
+			push(@{$groups{'table_data'}}, $1);
+			}
+		elsif ($priv =~ /^(create|drop|alter|create temp|create temporary|lock) tables$/) {
+			push(@{$groups{'tables'}}, $1);
+			}
+		elsif ($priv =~ /^(create|show) view$/) {
+			push(@{$groups{'view'}}, $1);
+			}
+		elsif ($priv =~ /^(create|alter) routine$/) {
+			push(@{$groups{'routine'}}, $1);
+			}
+		elsif ($priv =~ /^(slave|client) replication$/) {
+			push(@{$groups{'replication'}}, $1);
+			}
+		else {
+			push(@others, $priv);
+			}
+		}
+	# Simplify groups
+	my @simplified = ();
+	# Helper function to format group
+	my $format_group = sub {
+		my ($group, $suffix) = @_;
+		my $str = join(', ', @$group);
+		$str =~ s/(.*),/$1 $text{'dbs_except_and'}/ if (@$group > 1);
+		return $str . ($suffix ? " $suffix" : '');
+		};
+	# Handle each group
+	push(@simplified, $format_group->($groups{'table_data'}, 'table data'))
+		if (@{$groups{'table_data'}});
+	push(@simplified, $format_group->($groups{'tables'}, 'tables'))
+		if (@{$groups{'tables'}});
+	push(@simplified, $format_group->($groups{'view'}, 'view'))
+		if (@{$groups{'view'}});
+	push(@simplified, $format_group->($groups{'routine'}, 'routine'))
+		if (@{$groups{'routine'}});
+	push(@simplified, $format_group->($groups{'replication'}, 'replication'))	
+		if (@{$groups{'replication'}});
+	# Add other privileges
+	push(@simplified, @others);
+	return join('; ', @simplified);
+	};
+
+if (@privs_cur >= int(0.7 * @privs_all)) {
+	my %missing = map { $_ => 1 } @privs_all;
+	delete(@missing{@privs_cur});
+	my @missing_privs = keys %missing;
+	my $missing_formatted = $simplify_privs->(@missing_privs);
+	if (@missing_privs > 1) {
+		$privs_formatted = "$text{'dbs_except'} ($missing_formatted)";
+		}
+	else {
+		$privs_formatted = "$text{'dbs_except'} $missing_formatted";
+		}
+	}
+else {
+	$privs_formatted = $simplify_privs->(@privs_cur);
+	}
+return ucfirst($privs_formatted);
+}
+
 1;
 
