@@ -163,6 +163,24 @@ if ($key) {
 return $out;
 }
 
+# resolve_current_value(parameter_name)
+# Returns the value of the parameter, with compatibility-level conditionals
+sub resolve_current_value
+{
+my ($parameter_name) = @_;
+
+my $raw_value = &get_current_value($parameter_name);
+my $compatibility_level = &get_current_value("compatibility_level");
+
+if ($raw_value =~ /\{\{\$compatibility_level\}\s*([<>=!]+)\s*\{(\d+)\}\s*\?\s*\{(.*?)\}\s*:\s*\{(.*?)\}\}/) {
+	my ($op, $lvl, $tval, $fval) = ($1, $2, $3, $4);
+	return undef if ($op !~ /^([<>]=?|==|!=)$/);
+	return eval "\$compatibility_level $op $lvl" ? $tval : $fval;
+	}
+
+return $raw_value;
+}
+
 # if_default_value(parameter_name)
 # returns if the value is the default value
 sub if_default_value
@@ -224,11 +242,10 @@ sub set_current_value
     }
     else
     {
-        local ($out, $ex);
-	$ex = &execute_command(
+	my $out = &backquote_logged(
 		"$config{'postfix_config_command'} -c $config_dir ".
-		"-e $_[0]=".quotemeta($value), undef, \$out, \$out);
-	$ex && &error(&text('query_set_efailed', $_[0], $_[1], $out).
+		"-e $_[0]=".quotemeta($value)." 2>&1");
+	$? && &error(&text('query_set_efailed', $_[0], $_[1], $out).
 		      "<br> $config{'postfix_config_command'} -c $config_dir ".
 		     "-e $_[0]=\"$value\" 2>&1");
         &unflush_file_lines($config{'postfix_config_file'}); # Invalidate cache
@@ -814,6 +831,7 @@ sub get_maps
 		    &open_readfile(MAPS, $maps_file);
 		    my $i = 0;
 		    my $cmt;
+		    my $lastmap;
 		    while (<MAPS>)
 		    {
 			s/\r|\n//g;	# remove newlines
@@ -821,8 +839,8 @@ sub get_maps
 			    # A comment line
 			    $cmt = &is_table_comment($_);
 			    }
-			elsif (/^\s*(\/[^\/]*\/[a-z]*)\s+(.*)/ ||
-			       /^\s*([^\s]+)\s+(.*)/) {
+			elsif (/^(\/[^\/]*\/[a-z]*)\s+(.*)/ ||
+			       /^([^\s]+)\s+(.*)/) {
 			    # An actual map
 			    $number++;
 			    my %map;
@@ -837,6 +855,12 @@ sub get_maps
 			    $map{'cmt'} = $cmt;
 			    push(@{$maps_cache{$_[0]}}, \%map);
 			    $cmt = undef;
+			    $lastmap = \%map;
+			    }
+			elsif (/^\s+(\S.*$)/ && $lastmap) {
+			    # Continuation line
+			    $lastmap->{'value'} .= " ".$1;
+			    $lastmap->{'eline'} = $i;
 			    }
 			else {
 			    $cmt = undef;
@@ -2295,7 +2319,6 @@ return 1;
 sub supports_map_type
 {
 local ($type) = @_;
-return 1 if ($type eq 'hash');	# Assume always supported
 if (!scalar(@supports_map_type_cache)) {
 	@supports_map_type = ( );
 	open(POSTCONF, "$config{'postfix_config_command'} -m |");
