@@ -22,16 +22,17 @@ fi
 srcdir=$wadir
 ver=`cat "$wadir/version"`
 
-# Find temp directory
-if [ "$tempdir" = "" ]; then
-	tempdir=/tmp/.webmin
-fi
-
 if [ $? != "0" ]; then
 	echo "ERROR: Cannot find the Webmin install directory";
 	echo "";
 	exit 1;
 fi
+
+# Find temp directory
+if [ "$tempdir" = "" ]; then
+	tempdir=/tmp/.webmin
+fi
+
 echo "****************************************************************************"
 echo "           Welcome to the Webmin setup script, version $ver"
 echo "****************************************************************************"
@@ -1045,10 +1046,10 @@ if [ "$nostart" = "" ]; then
 		echo "Since Webmin was installed outside the package manager, ensure the"
 		echo "following recommended Perl modules and packages are present:"
 		echo " Perl modules:"
-		echo "  - DateTime, DateTime::Locale, DateTime::TimeZone, Data::Dumper"
-		echo "  - Digest::MD5, Digest::SHA, Encode::Detect, File::Basename"
-		echo "  - File::Path, Net::SSLeay, Time::HiRes, Time::Local, Time::Piece"
-		echo "  - lib, open"
+		echo "  - DateTime, DateTime::Locale, DateTime::TimeZone, Data::Dumper,"
+		echo "  - Digest::MD5, Digest::SHA, Encode::Detect, File::Basename,"
+		echo "  - File::Path, Net::SSLeay, Time::HiRes, Time::Local, Time::Piece,"
+		echo "  - JSON::XS, lib, open"
 		echo " Packages:"
 		echo "  - openssl - Cryptography library with TLS implementation"
 		echo "  - shared-mime-info - Shared MIME information database"
